# -*- coding: utf-8 -*-

from wallabag.api.api import Api, ApiMethod


class GetTagsForEntry(Api):

    def __init__(self, config, entry_id):
        Api.__init__(self, config)
        self.entry_id = entry_id

    def _get_api_url(self):
        entry_id = self._validate_identificator(self.entry_id)
        return self._build_url(ApiMethod.GET_TAGS_FOR_ENTRY).format(entry_id)

    def _make_request(self, request):
        return self._request_get(request)



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.get_tags_for_entry import GetTagsForEntry
from wallabag.config import Configs


class TestGetTagsForEntry():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    @pytest.mark.parametrize('entry_id', ["10", 10])
    def test_api_url(self, entry_id):
        api = GetTagsForEntry(self.config, entry_id)

        url = "url" + ApiMethod.GET_TAGS_FOR_ENTRY.value.format("10")
        assert url == api._get_api_url()

    @pytest.mark.parametrize('entry_id', ["-10", -10, None, "none"])
    def test_api_url_wrong_arg(self, entry_id):
        api = GetTagsForEntry(self.config, entry_id)

        with pytest.raises(ValueException):
            api._get_api_url()



# -*- coding: utf-8 -*-

from webbrowser import BaseBrowser

from click.testing import CliRunner
from wallabag.commands.open import OpenCommand, OpenCommandParams
from wallabag.config import Configs
from wallabag.api.get_entry import GetEntry
from wallabag.api.api import Response
from wallabag import wallabag


def config__is_valid(self):
    return True


class TestOpenCommand():

    runner = CliRunner()

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_open_command(self, monkeypatch):
        open_new_tab_runned = False

        def open_new_tab(self, url):
            nonlocal open_new_tab_runned
            open_new_tab_runned = True
            assert url == 'url/view/1'

        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "<h1>head</h1>content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)
        monkeypatch.setattr(BaseBrowser, 'open_new_tab', open_new_tab)

        result, msg = OpenCommand(
                self.config, OpenCommandParams(
                    1, browser='w3m')).execute()
        assert open_new_tab_runned
        assert result
        assert not msg

    def test_open_original(self, monkeypatch):
        open_new_tab_runned = False

        def open_new_tab(self, url):
            nonlocal open_new_tab_runned
            open_new_tab_runned = True
            assert url == 'url'

        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "<h1>head</h1>content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)
        monkeypatch.setattr(BaseBrowser, 'open_new_tab', open_new_tab)

        result, msg = OpenCommand(
                self.config, OpenCommandParams(
                    1, True, browser='w3m')).execute()
        assert open_new_tab_runned
        assert result
        assert not msg

    def test_command_open_simple(self, monkeypatch):
        command_runned = False

        def run_command(command, quiet=False):
            nonlocal command_runned
            command_runned = True
            assert command.__class__.__name__ == 'OpenCommand'
            assert command.params.entry_id == '1'
            assert not command.params.open_original

        monkeypatch.setattr(wallabag, 'run_command', run_command)
        monkeypatch.setattr(Configs, 'is_valid', config__is_valid)

        result = self.runner.invoke(
                wallabag.cli, ['open', '1', '-b', 'w3m'],
                catch_exceptions=False)
        assert command_runned
        assert result.exit_code == 0

    def test_command_open_original(self, monkeypatch):
        command_runned = False

        def run_command(command, quiet=False):
            nonlocal command_runned
            command_runned = True
            assert command.__class__.__name__ == 'OpenCommand'
            assert command.params.entry_id == '1'
            assert command.params.open_original

        monkeypatch.setattr(wallabag, 'run_command', run_command)
        monkeypatch.setattr(Configs, 'is_valid', config__is_valid)

        result = self.runner.invoke(
                wallabag.cli, ['open', '1', '-o', '-b', 'w3m'],
                catch_exceptions=False)
        assert command_runned
        assert result.exit_code == 0

    def test_command_browser(self, monkeypatch):
        command_runned = False

        def run_command(command, quiet=False):
            nonlocal command_runned
            command_runned = True
            assert command.__class__.__name__ == 'OpenCommand'
            assert command.params.entry_id == '1'
            assert command.params.browser == 'w3m'

        monkeypatch.setattr(wallabag, 'run_command', run_command)
        monkeypatch.setattr(Configs, 'is_valid', config__is_valid)

        result = self.runner.invoke(
                wallabag.cli, ['open', '1', '-b', 'w3m'],
                catch_exceptions=False)
        assert command_runned
        assert result.exit_code == 0



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.delete_tag_from_entry import DeleteTagFromEntry
from wallabag.api.delete_tag_by_id import DeleteTagsById
from wallabag.api.delete_tags_by_label import DeleteTagsByLabel
from wallabag.config import Configs


class TestDeleteTags():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_delete_from_entry_url(self):
        entry_id = 1
        tag_id = '2'
        api = DeleteTagFromEntry(self.config, entry_id, tag_id)
        url = 'url' + ApiMethod.DELETE_TAG_FROM_ENTRY.value.format(
                entry_id, tag_id)
        assert url == api._get_api_url()

    def test_delete_by_label(self):
        tag = 'tag'
        api = DeleteTagsByLabel(self.config, tag)
        url = 'url' + ApiMethod.DELETE_TAG_BY_LABEL.value
        assert url == api._get_api_url()
        assert DeleteTagsByLabel.ApiParams.TAGS.value in api._get_params()
        assert tag == api._get_params()[DeleteTagsByLabel.ApiParams.TAGS.value]

    def test_delete_by_id(self):
        tag_id = '10'
        api = DeleteTagsById(self.config, tag_id)
        url = 'url' + ApiMethod.DELETE_TAG_BY_ID.value.format(tag_id)
        assert url == api._get_api_url()

    @pytest.mark.parametrize('entry_id', ["-10", -10, None, "none"])
    def test_wrong_args(self, entry_id):
        api = DeleteTagFromEntry(self, entry_id, '10')
        with pytest.raises(ValueException):
            api._get_api_url()



# -*- coding: utf-8 -*-

import os
import tempfile

import pytest

from wallabag.api.api import Api, Response, RequestException
from wallabag.api.get_api_version import ApiVersion
from wallabag.api.api_token import ApiToken
from wallabag.config import Configs, Options, Sections
from wallabag.configurator import (
        ClientOption, Configurator, PasswordOption, SecretOption,
        ServerurlOption, UsernameOption, Validator)


def is_minimum_version(response):
    return True


def api_success(self):
    return Response(200, None)


def api_failure(self):
    raise RequestException(response=Response(403, None))


class TestConfigurator():

    fd = None

    def teardown_method(self, method):
        if self.fd:
            os.close(self.fd)
            os.remove(self.path)

    def setup_method(self, method):
        self.fd, self.path = tempfile.mkstemp()
        with open(self.path, 'w') as f:
            f.write('')
        self.configs = Configs(self.path)
        self.configs.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_init_configurator(self):
        c = Configurator(self.configs)
        assert c.config == self.configs

    def test_option_pass(self, monkeypatch):
        setup_runned = False

        def setup(self, config):
            nonlocal setup_runned
            setup_runned = True
            return True

        def save(self):
            pass

        monkeypatch.setattr(UsernameOption, 'setup', setup)
        monkeypatch.setattr(Configs, 'save', save)

        c = Configurator(self.configs)
        c.start([UsernameOption()])
        assert setup_runned

    @pytest.mark.parametrize(
            'values',
            [('wb.site', 'https://wb.site'),
             ('http://wb.site/', 'http://wb.site'),
             ('https://wb.site', 'https://wb.site')])
    def test_serverurl_option(self, monkeypatch, values):
        def testresponse(self):
            return False

        monkeypatch.setattr(Response, 'has_error', testresponse)
        monkeypatch.setattr(ApiVersion, 'request', api_success)
        monkeypatch.setattr(Api, 'is_minimum_version', is_minimum_version)

        so = ServerurlOption(self.configs)
        so.check_and_apply(values[0])

        assert values[1] == so.get_value()

    @pytest.mark.parametrize(
            'values',
            [(ServerurlOption(Api), (Sections.API, Options.SERVERURL)),
             (UsernameOption(), (Sections.API, Options.USERNAME)),
             (PasswordOption(), (Sections.API, Options.PASSWORD)),
             (ClientOption(), (Sections.OAUTH2, Options.CLIENT)),
             (SecretOption(), (Sections.OAUTH2, Options.SECRET))])
    def test_option_names(self, values):
        assert (values[1][0], values[1][1]) == values[0].get_option_name()

    def test_validator_check_success(self, monkeypatch):
        monkeypatch.setattr(ApiVersion, 'request', api_success)
        monkeypatch.setattr(ApiToken, 'request', api_success)
        monkeypatch.setattr(Api, 'is_minimum_version', is_minimum_version)

        (result, msg) = Validator(self.configs).execute()

        assert result
        assert msg == 'The config is suitable.'

    def test_validator_check_error(self, monkeypatch):
        monkeypatch.setattr(ApiVersion, 'request', api_failure)
        monkeypatch.setattr(ApiToken, 'request', api_failure)
        monkeypatch.setattr(Api, 'is_minimum_version', is_minimum_version)

        (result, msg) = Validator(self.configs).execute()

        assert not result
        assert msg == 'The server or the API is not reachable.'

    def test_validator_oauth_success(self, monkeypatch):
        monkeypatch.setattr(ApiToken, 'request', api_success)

        (result, msg, opts) = Validator(None).check_oauth()
        assert result
        assert msg == 'The configuration is ok.'

    def test_validator_oauth_failure(self, monkeypatch):
        monkeypatch.setattr(ApiToken, 'request', api_failure)

        (result, msg, opts) = Validator(None).check_oauth()
        assert not result
        assert not msg

    def test_validator_oauth_failure_bad_request_grant(self, monkeypatch):
        def api_failure_400_grant(self):
            raise RequestException(
                    response=Response(400, '{"error": "invalid_grant"}'))

        monkeypatch.setattr(ApiToken, 'request', api_failure_400_grant)

        (result, msg, options) = Validator(None).check_oauth()
        assert not result
        assert not msg
        assert len(options) == 2
        assert isinstance(options[0], UsernameOption)
        assert isinstance(options[1], PasswordOption)

    def test_validator_oauth_failure_bad_request_client(self, monkeypatch):
        def api_failure_400_client(self):
            raise RequestException(
                    response=Response(400, '{"error": "invalid_client"}'))

        monkeypatch.setattr(ApiToken, 'request', api_failure_400_client)

        (result, msg, options) = Validator(None).check_oauth()
        assert not result
        assert not msg
        assert len(options) == 2
        assert isinstance(options[0], ClientOption)
        assert isinstance(options[1], SecretOption)



import os
import tempfile
import pathlib

import pytest

from wallabag.config import Configs, Options, Sections

from xdg.BaseDirectory import xdg_config_home as XDG_CONFIG_HOME


class TestConfigs():

    configs = None

    def teardown_method(self, method):
        os.close(self.fd)
        os.remove(self.path)

    def setup_method(self, method):
        self.fd, self.path = tempfile.mkstemp()
        with open(self.path, 'w') as f:
            f.write('')
        self.configs = Configs(self.path)
        if method.__name__ in ['test_get_config', 'test_is_token_expired',
                               'test_set_config', 'test_set_config_new']:
            self.configs.config.read_string("""
                    [api]
                    serverurl = https://server
                    [token]
                    expires = 1000
                    """)
        elif method.__name__ == 'test_is_valid__true':
            self.configs.config.read_string("""
                    [api]
                    serverurl = url
                    username = user
                    password = pass
                    [oauth2]
                    client = 100
                    secret = 100
                    """)
        elif method.__name__ == 'test_get_path':
            self.configs = Configs()

    def test_get_path(self):
        xdg_config = os.path.expanduser(XDG_CONFIG_HOME)
        expected = f"{xdg_config}/wallabag-cli/config.ini"

        actual = self.configs.get_path()
        assert expected == str(actual)

    def test_get_path_custom(self):
        expected = pathlib.PurePath("custom/directory")

        assert expected == self.configs.get_path(expected)

    @pytest.mark.parametrize(
            'condition',
            [(Sections.TOKEN, Options.EXPIRES, '1000', 'get'),
             (Sections.TOKEN, Options.ACCESS_TOKEN, None, 'get'),
             (Sections.API, Options.SERVERURL, "https://server", 'get'),
             (Sections.API, '', 0, 'getint'),
             (Sections.API, '', None, 'get'),
             ('', '', None, 'get'),
             (None, None, None, 'get'),
             (None, None, 0, 'getint'),
             (Sections.TOKEN, Options.EXPIRES, 1000, 'getint')])
    def test_get_config(self, condition):
        if condition[3] == 'get':
            assert self.configs.get(condition[0], condition[1]) == condition[2]
        elif condition[3] == 'getint':
            assert self.configs.getint(
                    condition[0], condition[1]) == condition[2]

    def test_is_token_expired(self):
        assert self.configs.is_token_expired()

    def test_is_token_expired_no_value(self):
        assert self.configs.is_token_expired()

    def test_is_valid__false(self):
        assert not self.configs.is_valid()

    def test_is_valid__true(self):
        assert self.configs.is_valid()

    def test_set_config(self):
        self.configs.set(Sections.TOKEN, Options.EXPIRES, str(500))
        assert self.configs.getint(Sections.TOKEN, Options.EXPIRES) == 500

    def test_set_config_new(self):
        self.configs.set(Sections.TOKEN, Options.ACCESS_TOKEN, 'abba')
        assert self.configs.get(Sections.TOKEN, Options.ACCESS_TOKEN) == 'abba'

    def test_load_or_create(self, monkeypatch):
        self.save_called = False

        def exists(path):
            return False

        def savemock(configs, path):
            self.save_called = True
            return True

        monkeypatch.setattr(os.path, 'exists', exists)
        monkeypatch.setattr(Configs, 'save', savemock)
        self.configs.load_or_create()
        assert self.save_called

    def test_load_or_create_load(self, monkeypatch):
        self.load_called = False

        def exists(path):
            return True

        def loadmock(configs, path):
            self.load_called = True
            return True

        monkeypatch.setattr(os.path, 'exists', exists)
        monkeypatch.setattr(Configs, 'load', loadmock)
        self.configs.load_or_create()
        assert self.load_called

    def test_load_or_create_value_error(self, monkeypatch):
        def exists(path):
            return False

        def savemock(configs, path):
            return False

        monkeypatch.setattr(os.path, 'exists', exists)
        monkeypatch.setattr(Configs, 'save', savemock)
        with pytest.raises(ValueError, match=Configs.LOAD_ERROR):
            self.configs.load_or_create()

    @pytest.mark.parametrize(
            'password',
            ['123456', 'password', 'secret'])
    def test_encryption(self, password):
        self.configs.set(Sections.API, Options.PASSWORD, password)
        encrypted = self.configs.config.get(Sections.API, Options.PASSWORD)
        plain = self.configs.get(Sections.API, Options.PASSWORD)

        assert encrypted != password
        assert plain == password



# -*- coding: utf-8 -*-

import pytest

from colorama import Fore

from tags import tags_test
from wallabag.api.add_entry import AddEntry
from wallabag.api.api import Api, Response
from wallabag.api.entry_exists import EntryExists
from wallabag.commands.add import AddCommand, AddCommandParams
from wallabag.config import Configs


def get_authorization_header(self):
    return {'Authorization': "Bearer a1b2"}


class TestAddCommand():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_entry_existed_url(self, monkeypatch):
        make_request_runned = False
        url = "http://test/url"

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            assert request.api_params[EntryExists.ApiParams.URL.value] == url
            return Response(200, '{"exists": 1}')

        monkeypatch.setattr(EntryExists, '_make_request', _make_request)
        monkeypatch.setattr(
                Api, '_get_authorization_header', get_authorization_header)

        result = AddCommand(self.config, AddCommandParams(url)).execute()
        assert make_request_runned
        assert result[0]
        assert result[1] == "The url was already saved."

    def test_add_existed_url(self, monkeypatch):

        def entry_exists(self):
            return Response(200, '{"exists": 1}')

        monkeypatch.setattr(EntryExists, 'request', entry_exists)

        params = AddCommandParams('url')
        result = AddCommand(self.config, params).execute()
        assert result[0]
        assert result[1] == "The url was already saved."

    def test_add_command_params(self, monkeypatch):
        make_request_runned = False
        url = "http://test/url"
        title = 'test title'
        starred = True
        read = False

        def entry_not_exists(self):
            return Response(200, '{"exists": 0}')

        def _validate_url(self, url):
            return url

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            assert request.data[AddEntry.ApiParams.URL.value] == url
            assert request.data[AddEntry.ApiParams.ARCHIVE.value] == 0
            assert request.data[AddEntry.ApiParams.STARRED.value] == 1
            assert request.data[AddEntry.ApiParams.TITLE.value] == title
            return Response(200, (
                    '{"is_archived":0,"is_starred":0,"user_name":"wallabag",'
                    '"user_email":"","user_id":1,"tags":[],"is_public":false,'
                    '"id":15,"uid":null,"title":"title of entry",'
                    '"url":"url","content":"content",'
                    '"created_at":"2020-11-11T05:02:11+0000",'
                    '"updated_at":"2020-11-11T05:02:11+0000",'
                    '"published_at":null,"published_by":null,'
                    '"starred_at":null,"annotations":null}'
                ))

        monkeypatch.setattr(EntryExists, 'request', entry_not_exists)
        monkeypatch.setattr(AddEntry, '_make_request', _make_request)
        monkeypatch.setattr(AddEntry, '_validate_url', _validate_url)
        monkeypatch.setattr(
                Api, '_get_authorization_header', get_authorization_header)

        params = AddCommandParams(url, title, read=read, starred=starred)
        result = AddCommand(self.config, params).execute()
        assert make_request_runned
        assert result[0]
        assert result[1] == (
                "Entry successfully added:\n\n"
                f"\t{Fore.GREEN}15. title of entry{Fore.RESET}\n")

    @pytest.mark.parametrize('tags', tags_test)
    def test_add_with_tags(self, monkeypatch, tags):
        make_request_runned = False
        url = "http://test/url"
        title = 'test title'

        def entry_not_exists(self):
            return Response(200, '{"exists": 0}')

        def _validate_url(self, url):
            return url

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            assert request.data[AddEntry.ApiParams.URL.value] == url
            assert request.data[
                    AddEntry.ApiParams.TAGS.value].split(',') == tags[1]
            assert request.data[AddEntry.ApiParams.TITLE.value] == title
            return Response(200, (
                    '{"is_archived":0,"is_starred":0,"user_name":"wallabag",'
                    '"user_email":"","user_id":1,"tags":[],"is_public":false,'
                    '"id":15,"uid":null,"title":"title of entry",'
                    '"url":"url","content":"content",'
                    '"created_at":"2020-11-11T05:02:11+0000",'
                    '"updated_at":"2020-11-11T05:02:11+0000",'
                    '"published_at":null,"published_by":null,'
                    '"starred_at":null,"annotations":null,'
                    '"mimetype":"text/html","language":null,'
                    '"reading_time":7,"domain_name":"domain",'
                    '"preview_picture":"http://pic.com/pic.png",'
                    '"http_status":"200","headers":null,"origin_url":null,'
                    '"_links":{"self":{"href":"/api/entries/15"}}}'
                ))

        monkeypatch.setattr(EntryExists, 'request', entry_not_exists)
        monkeypatch.setattr(AddEntry, '_make_request', _make_request)
        monkeypatch.setattr(AddEntry, '_validate_url', _validate_url)
        monkeypatch.setattr(
                Api, '_get_authorization_header', get_authorization_header)

        params = AddCommandParams(url, title, tags=tags[0])
        result = AddCommand(self.config, params).execute()
        if tags[0]:
            assert make_request_runned
            assert result[0]
            assert result[1] == (
                    "Entry successfully added:\n\n"
                    f"\t{Fore.GREEN}15. title of entry{Fore.RESET}\n")
        else:
            assert not make_request_runned
            assert not result[0]
            assert result[1] == "tags value is empty"



# -*- coding: utf-8 -*-

from colorama import Fore

from wallabag.commands.info import InfoCommand, InfoCommandParams
from wallabag.config import Configs
from wallabag.api.api import Response
from wallabag.api.get_entry import GetEntry


class TestInfoCommand():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

        self.f_c = Fore.LIGHTBLUE_EX
        self.f_rst = Fore.RESET

    def test_info_success(self, monkeypatch):

        def request(self):
            return Response(
                    200, (
                     '{"id": 1, "title": "title", "content": "content",'
                     '"url": "url", "is_archived": 0, "is_starred": 1, '
                     '"tags": ['
                     '{"id": 0, "label": "tag1", "slug": "tag1"},'
                     '{"id": 1, "label": "tag2", "slug": "tag2"}],'
                     '"created_at": "2020-11-04T11:37:45+0000", '
                     '"updated_at": "2020-11-04T11:37:45+0000", '
                     '"published_at": null, "published_by":'
                     '["Publisher Name"],'
                     '"starred_at": null, "annotations": [], '
                     '"mimetype": "text/html", "language": "ru_RU",'
                     '"reading_time": 9, "domain_name": "www.site.com",'
                     '"preview_picture": "https://www.site.coms/pic.jpg"}'))

        monkeypatch.setattr(GetEntry, 'request', request)

        params = InfoCommandParams(entry_id=10)
        result, msg = InfoCommand(self.config, params).execute()
        assert result
        assert msg == (
                f'{self.f_c}ID{self.f_rst}: 1\n'
                f'{self.f_c}Title{self.f_rst}: title\n'
                f'{self.f_c}Url{self.f_rst}: url\n'
                f'{self.f_c}Tags{self.f_rst}: tag1 tag2\n'
                f'{self.f_c}Is read{self.f_rst}: False\n'
                f'{self.f_c}Is starred{self.f_rst}: True\n'
                f'{self.f_c}Created at{self.f_rst}: '
                'Apr 11, 2020, 11:37:45 AM\n'
                f'{self.f_c}Published by{self.f_rst}: Publisher Name\n'
                f'{self.f_c}Reading time{self.f_rst}: 9 min\n'
                f'{self.f_c}Preview picture{self.f_rst}: '
                'https://www.site.coms/pic.jpg')

    def test_info_with_anno(self, monkeypatch):

        def request(self):
            return Response(
                    200, (
                     '{"id": 1, "title": "title", "content": "content",'
                     '"url": "url", "is_archived": 0, "is_starred": 1, '
                     '"created_at": "2020-11-04T11:37:45+0000", '
                     '"updated_at": "2020-11-04T11:37:45+0000", '
                     '"starred_at": null, '
                     '"annotations": ['
                     '{"user": null, "annotator_schema _version": "v1.0", '
                     '"id": 3, "text": "Are you texting?", '
                     '"created_at": "2020-11-04T05:57:26+0000", '
                     '"updated_at": "2020-11-04T05:57:26+0000", '
                     '"quote": "Our current quote", '
                     '"ranges": ['
                     '{"start": "/p[5]", "startOffset": "0", "end": "/p[5]", '
                     '"endOffset": "19"}]},'
                     '{"user": null, "annotator_schema_version": "v1.0", '
                     '"id": 4, "text": "This is text", '
                     '"created_at": "2020-11-04T05:57:46+0000", '
                     '"updated_at": "2020-11-04T05:57:46+0000", '
                     '"quote": "Email Security", "ranges": ['
                     '{"start": "/h3[4]", "startOffset": "0", "end": "/h3[4]",'
                     '"endOffset": "14"}]}],'
                     '"mimetype": "text/html", "language": "ru_RU",'
                     '"reading_time": 9, "domain_name": "www.site.com"}'))

        monkeypatch.setattr(GetEntry, 'request', request)

        params = InfoCommandParams(entry_id=10)
        result, msg = InfoCommand(self.config, params).execute()
        assert result
        assert msg == (
                f'{self.f_c}ID{self.f_rst}: 1\n'
                f'{self.f_c}Title{self.f_rst}: title\n'
                f'{self.f_c}Url{self.f_rst}: url\n'
                f'{self.f_c}Annotations{self.f_rst}: 2\n'
                f'{self.f_c}Is read{self.f_rst}: False\n'
                f'{self.f_c}Is starred{self.f_rst}: True\n'
                f'{self.f_c}Created at{self.f_rst}: '
                'Apr 11, 2020, 11:37:45 AM\n'
                f'{self.f_c}Reading time{self.f_rst}: 9 min\n')

    def test_no_id(self):
        params = InfoCommandParams(None)
        result, msg = InfoCommand(self.config, params).execute()
        assert not result
        assert msg == 'Entry ID not specified'

        result, msg = InfoCommand(self.config, None).execute()
        assert not result
        assert msg == 'Entry ID not specified'



#!/usr/bin/env python3

import functools
import logging
import platform
import subprocess
import sys

from colorama import Fore

import click

from wallabag.commands.add import AddCommand, AddCommandParams
from wallabag.commands.anno import (
        AnnoCommand, AnnoSubcommand, AnnoCommandParams)
from wallabag.commands.delete import DeleteCommand, DeleteCommandParams
from wallabag.commands.list import ListCommand, ListParams, CountCommand
from wallabag.commands.show import ShowCommand, ShowCommandParams
from wallabag.commands.tags import (
        TagsCommand, TagsCommandParams, TagsSubcommand)
from wallabag.commands.open import OpenCommand, OpenCommandParams
from wallabag.commands.info import InfoCommand, InfoCommandParams
from wallabag.commands.update import UpdateCommand, UpdateCommandParams
from wallabag.config import Configs
from wallabag.configurator import (
        ClientOption,
        Configurator,
        PasswordOption,
        SecretOption,
        Validator,
    )
from wallabag.commands.update_by_tags import UpdateByTagsCommand
from wallabag.commands.delete_by_tags import DeleteByTags, DeleteByTagsParams
from wallabag import wclick

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


def __init_logging(debug, debug_level):
    log_level = logging.getLevelName(debug_level.upper())
    logging.basicConfig(
            level=logging.CRITICAL,
            format=(f'{Fore.YELLOW}%(msecs)d:%(name)s:%(levelname)s: '
                    f'%(message)s{Fore.RESET}'))

    logger = logging.getLogger('wallabag')
    if debug:
        logger.setLevel(log_level)
    else:
        logger.setLevel(logging.CRITICAL)


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option('--config', help='Use custom configuration file')
@click.option('--debug', is_flag=True, help='Enable debug logging to stdout')
@click.option('--debug-level', default='debug', help='Debug level')
@click.version_option(prog_name="wallabag")
@click.pass_context
def cli(ctx, config, debug, debug_level):
    __init_logging(debug, debug_level)

    # Workaround for default non-unicode encodings in the
    # Windows cmd and Powershell
    # -> Analyze encoding and set to utf-8
    if platform.system() == "Windows":
        codepage = subprocess.check_output(['chcp'], shell=True).decode()
        if "65001" not in codepage:
            subprocess.check_output(['chcp', '65001'], shell=True)

    ctx.obj = Configs(config)

    logger = logging.getLogger('wallabag')
    logger.info('wallabag started')


def need_config(func):
    @functools.wraps(func)
    @click.pass_context
    def wrapper(ctx, *args, **kwargs):
        if not ctx.obj.is_valid():
            i = input(
                """Could not find a valid config.
Would you like to create it now? [Y/n]
""")
            if str.lower(i) in ["y", "yes", ""]:
                Configurator(ctx.obj).start()
            else:
                sys.exit(0)
        func(*args, **kwargs)

    return wrapper


@cli.command()
@click.option('-s/-u', '--starred/--unstarred', default=None,
              help="Show only starred/unstarred entries.")
@click.option('-r/-n', '--read/--unread', default=None,
              help="Show only read/unread entries.")
@click.option('-a', '--all', default=False, is_flag=True,
              help="Show read as well as unread entries.")
@click.option('-o', '--oldest', default=False, is_flag=True,
              help="Show oldest matches instead of the newest.")
@click.option('-t', '--trim-output', default=False, is_flag=True,
              help="Trim the titles to fit the length of the cli.")
@click.option('-c', '--count', default=False, is_flag=True,
              help="Show a sum of matching entries.")
@click.option('-g', '--tags', help="Return entries that matches ALL tags.")
@click.option('-q', '--quantity', type=click.INT,
              help="Set the number of entries to show.")
@need_config
@click.pass_context
def list(ctx, starred, read, all, oldest, trim_output, count, tags, quantity):
    """
    List the entries on the wallabag account.

    Gives a summary of entries in wallabag. Use options to filter the results.
    """
    config = ctx.obj
    if all:
        read = None
        starred = None

    params = ListParams(quantity, read, starred, oldest, trim_output, tags)
    run_command(
            CountCommand(config, params) if count else
            ListCommand(config, params))


@cli.command()
@click.option('-c/-n', '--color/--no-color', default=True)
@click.option('-i', '--image-links', default=False, is_flag=True,
              help="Show image links in optimized output")
@click.option('-r', '--raw', default=False, is_flag=True,
              help="Disable wordwise trimming")
@click.option('-t', '--html', default=False, is_flag=True,
              help="Show the entry as html instead of optimized output for \
the cli.")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def show(ctx, entry_id, color, html, raw, image_links):
    """
    Show the text of an entry.

    The ENTRY_ID can be found with `list` command.
    """
    run_command(ShowCommand(
        ctx.obj, ShowCommandParams(entry_id, color, html, raw, image_links)))


@cli.command()
@click.option('-q', '--quiet', default=False, is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def read(ctx, entry_id, quiet):
    """
    Toggle the read-status of an existing entry.

    This is an alias for `update --toggle-read <ENTRY_ID>` command.

    The ENTRY_ID can be found with `list` command.
    """
    params = UpdateCommandParams(entry_id)
    params.toggle_read = True
    params.quiet = quiet
    run_command(UpdateCommand(ctx.obj, params))


@cli.command()
@click.option('-q', '--quiet', default=False, is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def star(ctx, entry_id, quiet):
    """
    Toggle the starred-status of an existing entry.

    This is an alias for `update --toggle-starred <ENTRY_ID>` command.

    The ENTRY_ID can be found with `list` command.
    """
    params = UpdateCommandParams(entry_id)
    params.toggle_star = True
    params.quiet = quiet
    run_command(UpdateCommand(ctx.obj, params))


@cli.command()
@click.option('-t', '--title', default="", help="Add a custom title.")
@click.option('-r', '--read', default=False, is_flag=True,
              help="Mark as read.")
@click.option('-s', '--starred', default=False, is_flag=True,
              help="Mark as starred.")
@click.option('-a', '--tags', help="Comma-separated list of tags")
@click.option('-q', '--quiet', default=False, is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('url', required=True)
@need_config
@click.pass_context
def add(ctx, url, title, read, starred, tags, quiet):
    """Add a new entry to wallabag."""
    params = AddCommandParams(url, title, read, starred, tags)
    run_command(AddCommand(ctx.obj, params), quiet)


@cli.command()
@click.option('-f', '--force', default=False, is_flag=True,
              help="Do not ask before deletion.")
@click.option('-q', '--quiet', default=False, is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def delete(ctx, entry_id, force, quiet):
    """
    Delete an entry from wallabag.

    The ENTRY_ID can be found with `list` command.
    """
    params = DeleteCommandParams(entry_id, force, quiet)
    run_command(DeleteCommand(ctx.obj, params))


@cli.command()
@click.option('-f', '--force', default=False, is_flag=True,
              help="Do not ask before deletion.")
@click.option('-q', '--quiet', default=False, is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('tags', required=True)
@need_config
@click.pass_context
def delete_by_tags(ctx, tags, force, quiet):
    """
    Delete entries from wallabag by tags.

    The TAGS can be found with `tags -c list` command.
    """
    params = DeleteByTagsParams(tags, force, quiet)
    run_command(DeleteByTags(ctx.obj, params), quiet=quiet)


@cli.command()
@click.option('-t', '--title', default="", help="Change the title.")
@click.option('-r', '--toggle-read', is_flag=True,
              help="Toggle the read status")
@click.option('-s', '--toggle-starred', is_flag=True,
              help="Toggle the starred status")
@click.option('--read/--unread', default=None, help="Set the read status")
@click.option('--starred/--unstarred', default=None,
              help="Set the starred status")
@click.option('-q', '--quiet', is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def update(ctx, entry_id, title, toggle_read, toggle_starred,
           read, starred, quiet):
    """
        Toggle the read or starred status or change the title
        of an existing entry.

        The ENTRY_ID can be found with `list` command.
    """
    params = UpdateCommandParams()
    params.new_title = title
    params.toggle_read = toggle_read
    params.toggle_star = toggle_starred
    params.set_read_state = read
    params.set_star_state = starred
    params.quiet = quiet
    run_command(
            UpdateCommand(ctx.obj, entry_id, params), quiet=quiet)


@cli.command()
@click.option('-r/-n', '--read/--unread', default=None,
              help="Set the read status")
@click.option('-s/-u', '--starred/--unstarred', default=None,
              help="Set the starred status")
@click.option('-f', '--force', is_flag=True,
              help="Do not ask before update.")
@click.option('-q', '--quiet', is_flag=True,
              help="Hide the output if no error occurs.")
@click.argument('tags', required=True)
@need_config
@click.pass_context
def update_by_tags(ctx, tags, read, starred, force, quiet):
    """
        Set the read or starred status of an existing entries
        selected by tags.

        The TAGS can be found with `tags -c list` command.
    """
    params = UpdateCommandParams(False)
    params.set_read_state = read
    params.set_star_state = starred
    params.force = force
    params.quiet = quiet
    run_command(
            UpdateByTagsCommand(ctx.obj, tags, params), quiet=quiet)


@cli.command(short_help="Retrieve and print all tags.")
@click.option('-c', '--command', default=TagsSubcommand.LIST.name,
              type=click.Choice(TagsSubcommand.list(), case_sensitive=False),
              help="Subcommand")
@click.option('-e', '--entry-id', type=int, help="ENTRY ID")
@click.option('-t', '--tags', help="TAGS for subcommands.")
@click.option('--tag-id', type=int,
              help="TAG_ID - used for removing tag by ID")
@need_config
@click.pass_context
def tags(ctx, command, entry_id, tags, tag_id):
    """
    Tag manipulation command.

    list (default) command: Retrieve and print tags in format:

    \b
    {id}. {slug}
    {id}. {slug}
    {id}. {slug}

    If ENTRY_ID specified, make action related to this entry.
    The ENTRY_ID can be found with `list` command.

    add command: Add tags to entry. ENTRY_ID and TAGS should be specified.

    remove command: Remove tag from entry. (ENTRY_ID and TAGS) or TAG_ID
    should be specified.
    """
    params = TagsCommandParams(entry_id=entry_id, tags=tags, tag_id=tag_id)
    params.configure(TagsSubcommand.get(command))
    run_command(TagsCommand(ctx.obj, params))


@cli.command(short_help="Annotation commands.")
@click.option('-c', '--command', default=AnnoSubcommand.LIST.name,
              type=click.Choice(AnnoSubcommand.list(), case_sensitive=False),
              help="Subcommand")
@click.option('-e', '--entry-id', type=int, help="ENTRY ID")
@click.option('-a', '--anno-id', type=int, help="ANNOTATION ID")
@need_config
@click.pass_context
def anno(ctx, command, entry_id, anno_id):
    """
    Annotations manipulation command.

    list (default) command: Retrieve and print annotations for specified entry:

    \b
    {id}. {quote} ({updated}) [{length}]
    {id}. {quote} ({updated}) [{length}]
    {id}. {quote} ({updated}) [{length}]
    """
    params = AnnoCommandParams()
    params.entry_id = entry_id
    params.anno_id = anno_id
    params.command = AnnoSubcommand.get(command)
    run_command(AnnoCommand(ctx.obj, params))


@cli.command(short_help="Get entry information.")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def info(ctx, entry_id):
    """
    Show entry information.
    """
    run_command(InfoCommand(ctx.obj, InfoCommandParams(entry_id)))


@cli.command(short_help="Open entry in browser.")
@click.option('-o', '--open-original', is_flag=True,
              help="Open original article")
@click.option('-b', '--browser', type=str, help="Use particular browser")
@click.argument('entry_id', required=True)
@need_config
@click.pass_context
def open(ctx, entry_id, open_original, browser):
    """
    Open entry in browser.

    The `browser` parameter should be one of this list:
    https://docs.python.org/3/library/webbrowser.html#webbrowser.register

    Example: open 10 -b w3m
    """
    run_command(
            OpenCommand(
                ctx.obj, OpenCommandParams(entry_id, open_original, browser)))


@cli.command(short_help="Start configuration.")
@click.option('-c', '--check', is_flag=True,
              help="Check the config for errors.")
@click.option('-p', '--password', is_flag=True,
              help="Change the wallabag password.")
@click.option('-o', '--oauth', is_flag=True,
              help="Change the wallabag client credentials.")
@click.pass_context
def config(ctx, check, password, oauth):
    config = ctx.obj
    if check:
        run_command(Validator(config))
        sys.exit(0)
    options = []
    if password or oauth:
        if not config.is_valid():
            wclick.echo(
                "Invalid existing config. "
                "Therefore you have to enter all values.")
        else:
            if password:
                options.append(PasswordOption())
            if oauth:
                options.append(ClientOption())
                options.append(SecretOption())
    configurator = Configurator(config)
    while True:
        with wclick.spinner():
            configurator.start(options)
            (result, msg, options) = Validator(config).check_oauth()

        if result or not options:
            wclick.echo(msg)
            sys.exit(0)


def run_command(command, quiet=False):
    with wclick.spinner():
        result, output = command.execute()

    if not quiet and output:
        click.echo(output)
    if not result:
        sys.exit(1)



# -*- coding: utf-8 -*-

import click
from colorama import Back

from wallabag.api.api import Response
from wallabag.api.delete_entry import DeleteEntry
from wallabag.api.get_entry import GetEntry
from wallabag.commands.delete import DeleteCommand, DeleteCommandParams
from wallabag.config import Configs


class TestDeleteCommand():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_not_confirmed_deletion(self, monkeypatch):
        def click_confirm(message):
            return False

        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)
        monkeypatch.setattr(click, 'confirm', click_confirm)

        params = DeleteCommandParams(1)
        result, output = DeleteCommand(self.config, params).execute()
        assert result
        assert output == 'Cancelling'

    def test_confirm_message(self, monkeypatch):
        confirm_runned = False

        def click_confirm(message):
            nonlocal confirm_runned
            confirm_runned = True
            assert message == (
                    f"{Back.RED}{DeleteCommand.WARN_MSG}:{Back.RESET}\n\n"
                    f"\ttitle\n\n"
                    f"Continue?")
            return False

        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)
        monkeypatch.setattr(click, 'confirm', click_confirm)

        params = DeleteCommandParams(1)
        result, output = DeleteCommand(self.config, params).execute()

        assert confirm_runned

    def test_deleted_successfully(self, monkeypatch):
        def click_confirm(message):
            return True

        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        def request_delete(self):
            return Response(200, None)

        monkeypatch.setattr(GetEntry, 'request', request)
        monkeypatch.setattr(DeleteEntry, 'request', request_delete)
        monkeypatch.setattr(click, 'confirm', click_confirm)

        params = DeleteCommandParams(1)
        result, output = DeleteCommand(self.config, params).execute()
        assert result
        assert output == 'Entry successfully deleted.'



# -*- coding: utf-8 -*-

import logging
from abc import ABC, abstractmethod
from wallabag.api.api import ApiException


class Command(ABC):

    params = None

    def __init__(self):
        self.log = logging.getLogger('wallabag.command')

    def execute(self):
        self.log.debug('executing command: %s', self.__class__.__name__)

        validated, msg = self.__validate()
        if not validated:
            return False, msg
        try:
            return self._run()
        except ApiException as ex:
            return False, str(ex)

    def __validate(self):
        if self.params:
            self.log.debug('validating parameters: %s', self.params.__dict__)

            return self.params.validate()
        return True, None

    @abstractmethod
    def _run(self):
        pass



# -*- coding: utf-8 -*-

from colorama import Fore

from wallabag.api.add_entry import AddEntry, Params as AddEntryParams
from wallabag.api.entry_exists import EntryExists
from wallabag.commands.command import Command
from wallabag.commands.tags_param import TagsParam
from wallabag.commands.params import Params
from wallabag.entry import Entry


class AddCommandParams(Params, TagsParam):
    target_url = None
    title = None
    starred = None
    read = None
    tags = None

    def __init__(self, target_url, title=None,
                 starred=None, read=None, tags=None):
        self.target_url = target_url
        self.title = title
        self.starred = starred
        self.read = read
        self.tags = tags

    def validate(self):
        return self._validate_tags()


class AddCommand(Command):

    def __init__(self, config, params=None):
        Command.__init__(self)
        self.config = config
        self.params = params

    def _run(self):
        params = self.params
        api = EntryExists(self.config, params.target_url)
        if api.request().response['exists']:
            return True, "The url was already saved."

        entry = Entry(AddEntry(self.config, params.target_url, {
            AddEntryParams.TITLE: params.title,
            AddEntryParams.READ: params.read,
            AddEntryParams.STARRED: params.starred,
            AddEntryParams.TAGS: params.tags
        }).request().response)
        return True, (
                "Entry successfully added:\n\n"
                f"\t{Fore.GREEN}{entry.entry_id}. {entry.title}{Fore.RESET}\n")



# -*- coding: utf-8 -*-

import click

from colorama import Back

from wallabag.config import Configs
from wallabag.api.api import Response
from wallabag.api.delete_entry import DeleteEntry
from wallabag.api.get_list_entries import GetListEntries
from wallabag.commands.delete_by_tags import DeleteByTags, DeleteByTagsParams


class TestDeleteByTags():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_remove_entries(self, monkeypatch):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def getlist_request(self):
            return Response(
                    200, (
                        '{"_embedded": {"items": [{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"tags": ['
                        '{"id":7,"label":"tag","slug":"tag"},'
                        '{"id":13,"label":"tag2","slug":"tag2"}]},'
                        '{"id": 2, "title": "title 2",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"tags": ['
                        '{"id":7,"label":"tag","slug":"tag"},'
                        '{"id":13,"label":"tag2","slug":"tag2"}]}]}}'))

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            assert msg == (
                    f'{Back.RED}You are going to remove '
                    f'followed entries:{Back.RESET}'
                    f'\n\n\ttitle\n\ttitle 2\n\nContinue?')
            return True

        monkeypatch.setattr(GetListEntries, 'request', getlist_request)
        monkeypatch.setattr(DeleteEntry, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = DeleteByTagsParams(tags='tag,tag2')
        result = DeleteByTags(self.config, params).execute()
        assert confirm_runned
        assert result[0]



# -*- coding: utf-8 -*-

from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.api.get_entry import GetEntry
from wallabag.api.update_entry import UpdateEntry, Params as UpdateEntryParams
from wallabag.entry import Entry


class UpdateCommandParams(Params):
    toggle_read = None
    toggle_star = None
    new_title = None
    set_read_state = None
    set_star_state = None
    force = False
    quiet = False
    check_toggle_options = True

    def __init__(self, check_toggle_options=True):
        self.check_toggle_options = check_toggle_options

    def validate(self):
        if self.set_star_state and self.toggle_star:
            self.toggle_star = None
        if self.set_read_state and self.toggle_read:
            self.toggle_read = None

        params = [
                not self.new_title,
                self.set_read_state is None,
                self.set_star_state is None
                ]
        if self.check_toggle_options:
            params.extend([not self.toggle_read, not self.toggle_star])

        for p in params:
            if not p:
                return True, None
        return False, 'No parameter given'


class UpdateCommand(Command):

    def __init__(self, config, entry_id, params):
        Command.__init__(self)
        self.config = config
        self.entry_id = entry_id
        self.params = params or UpdateCommandParams()

    def _run(self):
        params = self.params

        read_value = params.set_read_state
        star_value = params.set_star_state

        request = GetEntry(self.config, self.entry_id).request()
        entry = Entry(request.response)
        if params.toggle_read is not None and params.toggle_read:
            read_value = not entry.read
        if params.toggle_star is not None and params.toggle_star:
            star_value = not entry.starred

        request = UpdateEntry(self.config, self.entry_id, {
            UpdateEntryParams.TITLE: params.new_title,
            UpdateEntryParams.STAR: star_value,
            UpdateEntryParams.READ: read_value
        }).request()
        if not params.quiet:
            return True, "Entry successfully updated."
        return True, None



# -*- coding: utf-8 -*-

import click
import click_spinner

SPINNER = click_spinner.spinner()


def spinner():
    return SPINNER


def confirm(msg):
    SPINNER.stop()
    result = click.confirm(msg)
    SPINNER.start()
    return result


def echo(msg, *args, **kwargs):
    click.echo(msg, *args, **kwargs)


def prompt(msg, *args, **kwargs):
    SPINNER.stop()
    result = click.prompt(msg, *args, **kwargs)
    SPINNER.start()
    return result



# -*- coding: utf-8 -*-

import click
import pytest

from colorama import Back

from wallabag.config import Configs
from wallabag.api.api import Response
from wallabag.api.update_entry import UpdateEntry
from wallabag.api.get_list_entries import GetListEntries
from wallabag.commands.update import UpdateCommandParams
from wallabag.commands.update_by_tags import UpdateByTagsCommand


class TestUpdateByTags():

    entries_list_response = (
            '{"_embedded": {"items": [{"id": 1, "title": "title",'
            '"content": "<h1>head</h1>content", "url": "url",'
            '"is_archived": 0, "is_starred": 1,'
            '"tags": ['
            '{"id":7,"label":"tag","slug":"tag"},'
            '{"id":13,"label":"tag2","slug":"tag2"}]},'
            '{"id": 2, "title": "title 2",'
            '"content": "<h1>head</h1>content", "url": "url",'
            '"is_archived": 0, "is_starred": 1,'
            '"tags": ['
            '{"id":7,"label":"tag","slug":"tag"},'
            '{"id":13,"label":"tag2","slug":"tag2"}]}]}}')

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_update_entries(self, monkeypatch):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def getlist_request(self):
            return Response(200, TestUpdateByTags.entries_list_response)

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            assert msg == (
                    f'{Back.GREEN}You are going to update read status to read '
                    f'of followed entries:{Back.RESET}'
                    f'\n\n\ttitle\n\ttitle 2\n\nContinue?')
            return True

        monkeypatch.setattr(GetListEntries, 'request', getlist_request)
        monkeypatch.setattr(UpdateEntry, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = UpdateCommandParams(False)
        params.set_read_state = True
        result = UpdateByTagsCommand(self.config, 'tag,tag2', params).execute()
        assert confirm_runned
        assert result[0]

    @pytest.mark.parametrize('values', [
        ((True, False), (False, False), "read status to not read"),
        ((True, True), (False, False), "read status to read"),
        ((False, False), (True, False), "starred status to unstarred"),
        ((False, False), (True, True), "starred status to starred"),
        ((True, False), (True, True),
            "read status to not read and starred status to starred"),
        ((True, True), (True, True),
            "read status to read and starred status to starred"),
        ((True, True), (True, False),
            "read status to read and starred status to unstarred"),
        ])
    def test_confirm_messages(self, monkeypatch, values):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def getlist_request(self):
            return Response(200, TestUpdateByTags.entries_list_response)

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            assert msg == (
                    f'{Back.GREEN}You are going to update {values[2]} '
                    f'of followed entries:{Back.RESET}'
                    f'\n\n\ttitle\n\ttitle 2\n\nContinue?')
            return True

        monkeypatch.setattr(GetListEntries, 'request', getlist_request)
        monkeypatch.setattr(UpdateEntry, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = UpdateCommandParams(False)
        if values[0][0]:
            params.set_read_state = values[0][1]
        if values[1][0]:
            params.set_star_state = values[1][1]
        result = UpdateByTagsCommand(self.config, 'tag,tag2', params).execute()
        assert confirm_runned
        assert result[0]

    def test_force_param(self, monkeypatch):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def getlist_request(self):
            return Response(200, TestUpdateByTags.entries_list_response)

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            return True

        monkeypatch.setattr(GetListEntries, 'request', getlist_request)
        monkeypatch.setattr(UpdateEntry, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = UpdateCommandParams(False)
        params.set_read_state = True
        params.force = True
        result = UpdateByTagsCommand(self.config, 'tag,tag2', params).execute()
        assert not confirm_runned
        assert result[0]



# -*- coding: utf-8 -*-

import delorean
import humanize

from wallabag.config import Configs
from wallabag.api.api import Response
from wallabag.api.get_entry import GetEntry
from wallabag.api.delete_annotation import DeleteAnnotation
from wallabag.commands.anno import (
        AnnoCommand, AnnoCommandParams, AnnoSubcommand)


class TestAnno():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_list_entry_annotations(self, monkeypatch):

        def getentry_request(self):
            return Response(
                    200, (
                        '{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"annotations": [{'
                        '"user": "User", "annotator_schema_version":'
                        ' "v1.0", "id": 1, "text": "content", '
                        '"created_at": "2020-10-28T10:50:51+0000", '
                        '"updated_at": "2020-10-28T10:50:51+0000", '
                        '"quote": "quote", "ranges": '
                        '[{"start": "/div[1]/p[1]", "startOffset": "23", '
                        '"end": "/div[1]/p[1]", "endOffset": "49"}]}]}'))

        monkeypatch.setattr(GetEntry, 'request', getentry_request)

        params = AnnoCommandParams()
        params.entry_id = 1
        result = AnnoCommand(self.config, params).execute()
        assert result[0]
        past = delorean.utcnow() - delorean.parse('2020-10-28T10:50:51+0000')
        assert result[1] == f'1. quote ({humanize.naturaltime(past)}) [7]'

    def test_remove_annotation(self, monkeypatch):

        def request_success(self):
            return Response(200, None)

        monkeypatch.setattr(DeleteAnnotation, 'request', request_success)

        params = AnnoCommandParams()
        params.anno_id = 1
        params.command = AnnoSubcommand.REMOVE
        result = AnnoCommand(self.config, params).execute()
        assert result[0]
        assert result[1] == 'Annotation successfully deleted'

    def test_show_annotations(self, monkeypatch):

        def getentry_request(self):
            return Response(
                    200, (
                        '{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"annotations": [{'
                        '"user": "User", "annotator_schema_version":'
                        ' "v1.0", "id": 1, "text": "content", '
                        '"created_at": "2020-10-28T10:50:51+0000", '
                        '"updated_at": "2020-10-28T10:50:51+0000", '
                        '"quote": "quote", "ranges": '
                        '[{"start": "/div[1]/p[1]", "startOffset": "23", '
                        '"end": "/div[1]/p[1]", "endOffset": "49"}]},{'
                        '"user": "User", "annotator_schema_version":'
                        ' "v1.0", "id": 2, "text": "another content", '
                        '"created_at": "2020-10-28T10:50:51+0000", '
                        '"updated_at": "2020-10-28T10:50:51+0000", '
                        '"quote": "another quote", "ranges": '
                        '[{"start": "/div[1]/p[2]", "startOffset": "23", '
                        '"end": "/div[1]/p[2]", "endOffset": "49"}]}]}'))

        monkeypatch.setattr(GetEntry, 'request', getentry_request)

        params = AnnoCommandParams()
        params.entry_id = 1
        params.command = AnnoSubcommand.SHOW
        result = AnnoCommand(self.config, params).execute()
        assert result[0]
        past = delorean.utcnow() - delorean.parse('2020-10-28T10:50:51+0000')
        assert result[1] == (
                f'1. quote ({humanize.naturaltime(past)}):\n\n\tcontent\n\n'
                f'2. another quote ({humanize.naturaltime(past)}):'
                '\n\n\tanother content\n')

    def test_show_annotations_by_id(self, monkeypatch):

        def getentry_request(self):
            return Response(
                    200, (
                        '{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"annotations": [{'
                        '"user": "User", "annotator_schema_version":'
                        ' "v1.0", "id": 1, "text": "content", '
                        '"created_at": "2020-10-28T10:50:51+0000", '
                        '"updated_at": "2020-10-28T10:50:51+0000", '
                        '"quote": "quote", "ranges": '
                        '[{"start": "/div[1]/p[1]", "startOffset": "23", '
                        '"end": "/div[1]/p[1]", "endOffset": "49"}]},{'
                        '"user": "User", "annotator_schema_version":'
                        ' "v1.0", "id": 2, "text": "another content", '
                        '"created_at": "2020-10-28T10:50:51+0000", '
                        '"updated_at": "2020-10-28T10:50:51+0000", '
                        '"quote": "another quote", "ranges": '
                        '[{"start": "/div[1]/p[2]", "startOffset": "23", '
                        '"end": "/div[1]/p[2]", "endOffset": "49"}]}]}'))

        monkeypatch.setattr(GetEntry, 'request', getentry_request)

        params = AnnoCommandParams()
        params.entry_id = 1
        params.anno_id = 2
        params.command = AnnoSubcommand.SHOW
        result = AnnoCommand(self.config, params).execute()
        assert result[0]
        past = delorean.utcnow() - delorean.parse('2020-10-28T10:50:51+0000')
        assert result[1] == (
                f'2. another quote ({humanize.naturaltime(past)}):'
                '\n\n\tanother content\n')

    def test_show_empty_params(self):
        params = AnnoCommandParams()
        params.command = AnnoSubcommand.SHOW
        result = AnnoCommand(self.config, params).execute()
        assert not result[0]
        assert result[1] == 'Entry ID not specified'

    def test_list_empty_params(self):
        params = AnnoCommandParams()
        params.command = AnnoSubcommand.LIST
        result = AnnoCommand(self.config, params).execute()
        assert not result[0]
        assert result[1] == 'Entry ID not specified'

    def test_remove_empty_params(self):
        params = AnnoCommandParams()
        params.command = AnnoSubcommand.REMOVE
        result = AnnoCommand(self.config, params).execute()
        assert not result[0]
        assert result[1] == 'Annotation ID not specified'



# -*- coding: utf-8 -*-

from enum import Enum, auto

import delorean
import humanize

from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.api.get_entry import GetEntry
from wallabag.api.delete_annotation import DeleteAnnotation
from wallabag.entry import Entry


class AnnoSubcommand(Enum):
    LIST = auto()
    REMOVE = auto()
    SHOW = auto()

    def list():
        return [c.name for c in AnnoSubcommand]

    def get(name):
        for command in AnnoSubcommand:
            if command.name == name.upper():
                return command
        return AnnoSubcommand.LIST


class AnnoCommandParams(Params):
    command = AnnoSubcommand.LIST
    entry_id = None
    anno_id = None

    def validate(self):
        if self.command in (AnnoSubcommand.LIST, AnnoSubcommand.SHOW):
            if not self.entry_id:
                return False, 'Entry ID not specified'
        elif self.command == AnnoSubcommand.REMOVE:
            if not self.anno_id:
                return False, 'Annotation ID not specified'
        return True, None


class AnnoCommand(Command):

    def __init__(self, config, params):
        Command.__init__(self)
        self.config = config
        self.params = params

        self.__get_anno = {
                AnnoSubcommand.LIST: self.__get_anno_string,
                AnnoSubcommand.SHOW: self.__get_anno_full
        }

    def _run(self):
        params = self.params
        if params.command in (AnnoSubcommand.LIST, AnnoSubcommand.SHOW):
            api = GetEntry(self.config, params.entry_id)
            entry = Entry(api.request().response)
            result = []
            for anno in sorted(
                    entry.annotations, key=lambda x: int(x['id'])):
                result.append(self.__get_anno[params.command](anno))
            return True, "\n".join(filter(None, result))
        if params.command == AnnoSubcommand.REMOVE:
            DeleteAnnotation(self.config, self.params.anno_id).request()
            return True, 'Annotation successfully deleted'
        return False, "Unknown command"

    def __get_anno_string(self, anno):
        past = delorean.utcnow() - delorean.parse(anno['updated_at'])
        return (f"{anno['id']}. {anno['quote']} "
                f"({humanize.naturaltime(past)}) [{len(anno['text'])}]")

    def __get_anno_full(self, anno):
        if self.params.anno_id:
            if self.params.anno_id != int(anno['id']):
                return ""
        past = delorean.utcnow() - delorean.parse(anno['updated_at'])
        return (f"{anno['id']}. {anno['quote']} "
                f"({humanize.naturaltime(past)}):\n\n\t{anno['text']}\n")



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.delete_annotation import DeleteAnnotation
from wallabag.config import Configs


class TestDeleteAnnotations():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_url(self):
        anno_id = 1
        api = DeleteAnnotation(self.config, anno_id)
        url = 'url' + ApiMethod.DELETE_ANNOTATION.value.format(anno_id)
        assert url == api._get_api_url()

    @pytest.mark.parametrize('anno_id', ["-10", -10, None, "none"])
    def test_wrong_args(self, anno_id):
        api = DeleteAnnotation(self, anno_id)
        with pytest.raises(ValueException):
            api._get_api_url()



import re
import json
import logging

from abc import ABC, abstractmethod
from enum import auto, Enum
from wallabag.config import Options, Sections

import requests

from packaging import version

MINIMUM_API_VERSION = "2.1.1"


class ApiException(Exception):

    error_text = None
    error_description = None

    def __init__(self, error_text=None, error_description=None):
        self.error_text = error_text
        self.error_description = error_description

    def __str__(self):
        desc = f"- {self.error_description}" if self.error_description else ""
        return f"Error: {self.error_text}{desc}"


class OAuthException(ApiException):
    pass


class RequestException(ApiException):

    def __init__(self, text=None, description=None, response=None):
        if response:
            ApiException.__init__(
                    self, response.error_text, response.error_description)
        else:
            ApiException.__init__(
                    self, text, description)
        self.response = response


class ValueException(ApiException):
    pass


class UnsupportedMethodException(ApiException):

    def __init__(self, method_name):
        self.error_text = 'Unsupported method called'
        self.error_description = f'Method: {method_name}'


class Error(Enum):
    UNDEFINED = -1
    OK = 0
    DNS_ERROR = 1
    HTTP_BAD_REQUEST = 400
    HTTP_UNAUTHORIZED = 401
    HTTP_FORBIDDEN = 403
    HTTP_NOT_FOUND = 404
    METHOD_NOT_ALLOWED = 405
    UNKNOWN_ERROR = 999


class ApiMethod(Enum):
    ADD_ENTRY = "/api/entries"
    DELETE_ENTRY = "/api/entries/{0}"
    GET_ENTRY = "/api/entries/{0}"
    UPDATE_ENTRY = "/api/entries/{0}"
    ENTRY_EXISTS = "/api/entries/exists"
    LIST_ENTRIES = "/api/entries"
    ADD_TAGS_TO_ENTRY = "/api/entries/{0}/tags"
    GET_TAGS = "/api/tags"
    GET_TAGS_FOR_ENTRY = "/api/entries/{0}/tags"
    DELETE_TAG_FROM_ENTRY = "/api/entries/{0}/tags/{1}"
    DELETE_TAG_BY_ID = "/api/tags/{0}"
    DELETE_TAG_BY_LABEL = "/api/tags/label"
    DELETE_ANNOTATION = "/api/annotations/{0}"
    TOKEN = "/oauth/v2/token"
    VERSION = "/api/version"


class Verbs(Enum):
    GET = auto()
    POST = auto()
    PUT = auto()
    DELETE = auto()
    PATCH = auto()
    HEAD = auto()


class Response:
    error = Error.UNDEFINED
    error_text = ""
    error_description = ""

    response = None

    def __init__(self, status_code, text=None):
        if text:
            try:
                self.response = json.loads(text)
            except json.decoder.JSONDecodeError as err:
                self.error_text = str(err)
                self.response = text
        errors = {
            0: (Error.DNS_ERROR, ("Name or service not known.", None)),
            400: (Error.HTTP_BAD_REQUEST, self.__error_from_server),
            401: (Error.HTTP_UNAUTHORIZED, self.__error_from_server),
            403: (Error.HTTP_FORBIDDEN,
                  ("403: Could not reach API due to rights issues.", None)),
            404: (Error.HTTP_NOT_FOUND, ("404: API was not found.", None)),
            405: (Error.METHOD_NOT_ALLOWED,
                  ("405: Method not allowed.", None)),
            200: (Error.OK, None),
            418: (Error.OK, None),
        }

        if status_code in errors:
            result = errors.get(status_code)
        else:
            result = (Error.UNKNOWN_ERROR,
                      (f"An unknown error occured. {status_code}", None))

        self.error = result[0]
        if isinstance(result[1], tuple):
            self.error_text = result[1][0]
            self.error_description = result[1][1]
        elif result[1]:
            (self.error_text, self.error_description) = result[1]()

    def has_error(self):
        return self.error != Error.OK

    def __error_from_server(self):
        return (self.response['error'] if 'error' in self.response else None,
                self.response['error_description']
                if 'error_description' in self.response else None)


class Api(ABC):

    VERSION_RE = re.compile('\\d+\\.\\d+\\.\\d+')
    URL_RE = re.compile("(?i)https?:\\/\\/.+")
    HEAD_UA = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
    (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36'
    REQUEST_METHODS = {
        Verbs.GET: requests.get,
        Verbs.DELETE: requests.delete,
        Verbs.POST: requests.post,
        Verbs.PATCH: requests.patch,
        Verbs.HEAD: requests.head
    }

    class Request:
        type = None
        url = None
        api_params = None
        headers = None
        data = None

    skip_auth = False

    def __init__(self, config):
        self.log = logging.getLogger('wallabag.api')
        self.config = config

    def request(self):
        self.log.debug('making api request: %s', self.__class__.__name__)
        request = Api.Request()
        request.url = self._get_api_url()
        if not self.skip_auth:
            request.headers = self._get_authorization_header()
        request.api_params = self._get_params()
        request.data = self._get_data()
        return self._make_request(request)

    def is_minimum_version(version_response):
        versionstring = version_response.response

        if not Api.VERSION_RE.match(versionstring):
            return False

        ver = versionstring.strip('"')
        return version.parse(MINIMUM_API_VERSION) <= version.parse(ver)

    def _build_url(self, api_method, url=None):
        try:
            if api_method in ApiMethod:
                serverurl = url or self.config.get(
                        Sections.API, Options.SERVERURL)
                return serverurl + api_method.value
        except TypeError:
            raise UnsupportedMethodException(api_method)

    def _request_delete(self, request):
        request.type = Verbs.DELETE
        return self.__make_request(request)

    def _request_head(self, request):
        request.type = Verbs.HEAD
        return self.__make_request(request)

    def _request_get(self, request):
        request.type = Verbs.GET
        return self.__make_request(request)

    def _request_post(self, request):
        request.type = Verbs.POST
        return self.__make_request(request)

    def _request_patch(self, request):
        request.type = Verbs.PATCH
        return self.__make_request(request)

    def _is_valid_url(self, url):
        try_method_get = False
        while True:
            request = Api.Request()
            request.url = url
            request.headers = {'user-agent': Api.HEAD_UA}
            if not try_method_get:
                result = self._request_head(request)
                if result.error == Error.METHOD_NOT_ALLOWED:
                    try_method_get = True
                    continue
            else:
                result = self._request_get(request)
            break
        return not result.has_error()

    def _validate_url(self, url):
        if not url:
            raise ValueException("Invalid url")
        if not Api.URL_RE.match(url):
            for protocol in "https://", "http://":
                if self._is_valid_url(f"{protocol}{url}"):
                    url = f"{protocol}{url}"
                    valid_url = True
                    break
        else:
            valid_url = self._is_valid_url(url)

        if not valid_url:
            raise ValueException("Invalid url")
        return url

    def _validate_identificator(self, entry_id):
        if not entry_id:
            raise ValueException("ENTRY_ID is not a number")

        try:
            entry_id = int(entry_id)
        except ValueError:
            raise ValueException("ENTRY_ID is not a number")

        if entry_id < 0:
            raise ValueException("ENTRY_ID is less than zero")

        return entry_id

    def _put_bool_param(self, api_params, param, api_param):
        if self.params[param] is not None:
            if isinstance(self.params[param], bool):
                api_params[api_param.value] = 1 if self.params[param] else 0

    @abstractmethod
    def _make_request(self, request):
        pass

    @abstractmethod
    def _get_api_url(self):
        pass

    def _get_params(self):
        return None

    def _get_data(self):
        return None

    def _get_authorization_header(self):
        from wallabag.configurator import TokenConfigurator

        token = TokenConfigurator(self.config).get_token()
        return {'Authorization': f"Bearer {token}"}

    def __make_request(self, request):
        try:
            self.log.debug('request data: %s', request.__dict__)

            result = Api.REQUEST_METHODS[request.type](
                    request.url, headers=request.headers,
                    params=request.api_params, data=request.data,
                    allow_redirects=True)
            response = Response(result.status_code, result.text)
        except (
                requests.exceptions.ConnectionError,
                requests.exceptions.MissingSchema) as error:
            self.log.exception('request exception')
            raise RequestException(
                    'Connection error', error)

        self.log.debug('response result: %s', response.__dict__)

        if response.has_error():
            raise RequestException(response=response)
        return response



# -*- coding: utf-8 -*-

import pytest
from colorama import Fore
from tabulate import tabulate

from click.testing import CliRunner
from wallabag.api.api import Api, Response
from wallabag.api.get_list_entries import GetListEntries
from wallabag.commands.list import ListCommand, ListParams
from wallabag.config import Configs
from wallabag import wallabag
from tags import tags_test


def get_authorization_header(self):
    return {'Authorization': "Bearer a1b2"}


def config__is_valid(self):
    return True


class TestListCommand():

    runner = CliRunner()

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_empty_list(self, monkeypatch):

        def list_entries(self):
            text = '{ "_embedded": { "items": [] } }'
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, 'request', list_entries)

        command = ListCommand(self.config)
        result, entries = command.execute()
        assert result
        assert not entries

    @pytest.mark.parametrize('values', [
        ((0, 0), [1, '', 'title']), ((0, 1), [1, '', 'title']),
        ((1, 0), [1, '', 'title']), ((1, 1), [1, '', 'title']),
        ])
    def test_entries_list(self, monkeypatch, values):

        def list_entries(self):
            text = '''
            { "_embedded": { "items": [
                {
                "id": 1,
                "title": "title",
                "content": "content",
                "url": "url",
                "is_archived": %s,
                "is_starred": %s
                }
            ] } }
            ''' % (values[0][0], values[0][1])
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, 'request', list_entries)

        command = ListCommand(self.config)
        result, entries = command.execute()
        assert result
        assert entries
        assert entries == tabulate([values[1]])

    def test_list(self, monkeypatch):

        def list_entries(self):
            text = '''
            { "_embedded": { "items": [
                { "id": 1, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1},
                { "id": 2, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1}
            ] } }
            '''
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, 'request', list_entries)

        command = ListCommand(self.config)
        result, entries = command.execute()
        assert result
        assert len(entries.split('\n')) == 4

    @pytest.mark.parametrize('tags', tags_test)
    def test_tags_param(self, monkeypatch, tags):
        make_request_runned = False

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            assert GetListEntries.ApiParams.TAGS.value in request.api_params
            assert request.api_params[
                    GetListEntries.ApiParams.TAGS.value].split(',') == tags[1]
            text = '{ "_embedded": { "items": [] } }'
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, '_make_request', _make_request)
        monkeypatch.setattr(
                Api, '_get_authorization_header', get_authorization_header)

        command = ListCommand(self.config, ListParams(tags=tags[0]))
        result, entries = command.execute()
        if tags[0]:
            assert make_request_runned
            assert result
        else:
            assert not make_request_runned
            assert not result

    def test_annotated_entry(self, monkeypatch):

        def list_entries(self):
            text = '''
            { "_embedded": { "items": [
                { "id": 2, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1,
                "annotations": [{
                    "user": "User", "annotator_schema_version":
                    "v1.0", "id": 1, "text": "content",
                    "created_at": "2020-10-28T10:50:51+0000",
                    "updated_at": "2020-10-28T10:50:51+0000",
                    "quote": "quote", "ranges":
                    [{"start": "/div[1]/p[1]", "startOffset": "23",
                    "end": "/div[1]/p[1]", "endOffset": "49"}]}]},
                { "id": 1, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1}]}}
            '''
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, 'request', list_entries)

        command = ListCommand(self.config)
        result, entries = command.execute()
        assert result
        assert entries == tabulate(
                [
                    [2, '', f'title {Fore.BLUE}{Fore.RESET}'],
                    [1, '', 'title']])

    def test_tagged_entry(self, monkeypatch):

        def list_entries(self):
            text = '''
            { "_embedded": { "items": [
                { "id": 2, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1},
                { "id": 1, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1,
                "tags": [
                        {"id":7,"label":"tag","slug":"tag"},
                        {"id":13,"label":"tag2","slug":"tag2"}]}
                ]}}
            '''
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, 'request', list_entries)

        command = ListCommand(self.config)
        result, entries = command.execute()
        assert result
        assert entries == tabulate(
                [
                    [2, '', 'title'],
                    [1, '', f'title {Fore.BLUE}{Fore.RESET}']])

    def test_tagged_and_annotated_entry(self, monkeypatch):

        def list_entries(self):
            text = '''
            { "_embedded": { "items": [
                { "id": 2, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1},
                { "id": 1, "title": "title", "content": "content",
                "url": "url", "is_archived": 0, "is_starred": 1,
                "annotations": [{
                    "user": "User", "annotator_schema_version":
                    "v1.0", "id": 1, "text": "content",
                    "created_at": "2020-10-28T10:50:51+0000",
                    "updated_at": "2020-10-28T10:50:51+0000",
                    "quote": "quote", "ranges":
                    [{"start": "/div[1]/p[1]", "startOffset": "23",
                    "end": "/div[1]/p[1]", "endOffset": "49"}]}],
                "tags": [
                        {"id":7,"label":"tag","slug":"tag"},
                        {"id":13,"label":"tag2","slug":"tag2"}]}
                ]}}
            '''
            return Response(200, text)

        monkeypatch.setattr(GetListEntries, 'request', list_entries)

        command = ListCommand(self.config)
        result, entries = command.execute()
        assert result
        assert entries == tabulate(
                [
                    [2, '', 'title'],
                    [1, '', (
                        f'title {Fore.BLUE}{Fore.RESET} '
                        f'{Fore.BLUE}{Fore.RESET}')]])

    @pytest.mark.parametrize(
            'command_class', [
                (['list'], 'ListCommand'),
                (['list', '-c'], 'CountCommand')])
    def test_list_command(self, monkeypatch, command_class):
        command_runned = False

        def run_command(command, quite=False):
            nonlocal command_runned
            command_runned = True
            assert command.__class__.__name__ == command_class[1]

        monkeypatch.setattr(wallabag, 'run_command', run_command)
        monkeypatch.setattr(Configs, 'is_valid', config__is_valid)

        result = self.runner.invoke(
                wallabag.cli, command_class[0], catch_exceptions=False)
        assert command_runned
        assert result.exit_code == 0



# -*- coding: utf-8 -*-

from colorama import Back

from wallabag.api.get_entry import GetEntry
from wallabag.api.delete_entry import DeleteEntry
from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.entry import Entry
from wallabag import wclick


class DeleteCommandParams(Params):

    def __init__(self, entry_id, force=False, quiet=False):
        self.entry_id = entry_id
        self.force = force
        self.quiet = quiet


class DeleteCommand(Command):

    WARN_MSG = 'You are going to remove the following entry'

    def __init__(self, config, params):
        Command.__init__(self)
        self.config = config
        self.params = params or DeleteCommandParams()

    def _run(self):
        if not self.params.force:
            request = GetEntry(self.config, self.params.entry_id).request()
            entr = Entry(request.response)
            confirm_msg = (
                    f"{Back.RED}{DeleteCommand.WARN_MSG}:{Back.RESET}\n\n"
                    f"\t{entr.title}\n\n"
                    "Continue?")
            if not wclick.confirm(confirm_msg):
                return True, 'Cancelling'

        request = DeleteEntry(self.config, self.params.entry_id).request()
        if not self.params.quiet:
            return True, "Entry successfully deleted."
        return True, None



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.update_entry import UpdateEntry, Params
from wallabag.config import Configs


class TestUpdateEntry():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    @pytest.mark.parametrize('entry_id', [10, "10"])
    def test_api_url(self, entry_id):
        api = UpdateEntry(self.config, entry_id, {})

        url = "url" + ApiMethod.UPDATE_ENTRY.value.format(entry_id)
        assert url == api._get_api_url()

    @pytest.mark.parametrize('values', [
        (Params.TITLE, "new title", UpdateEntry.ApiParams.TITLE, "new title"),
        (Params.READ, True, UpdateEntry.ApiParams.ARCHIVE, 1),
        (Params.READ, False, UpdateEntry.ApiParams.ARCHIVE, 0),
        (Params.STAR, True, UpdateEntry.ApiParams.STARRED, 1),
        (Params.STAR, False, UpdateEntry.ApiParams.STARRED, 0),
        ])
    def test_params(self, values):
        api = UpdateEntry(self.config, "10", {
            values[0]: values[1]
        })

        data = api._get_data()
        assert data[values[2].value] == values[3]

    def test_empty_params(self):
        api = UpdateEntry(self.config, "10", {
            Params.TITLE: "",
            Params.STAR: None
        })

        with pytest.raises(ValueException):
            api._get_data()

    def test_mixed_params(self):
        api = UpdateEntry(self.config, "10", {
            Params.TITLE: "",
            Params.READ: True
        })

        data = api._get_data()
        assert UpdateEntry.ApiParams.TITLE.value not in data
        assert data[UpdateEntry.ApiParams.ARCHIVE.value] == 1



# -*- coding: utf-8 -*-

from enum import Enum, auto

from colorama import Fore, Back

from wallabag.api.add_tag_to_entry import AddTagToEntry, Params as AddTagParams
from wallabag.api.get_tags import GetTags
from wallabag.api.delete_tag_from_entry import DeleteTagFromEntry
from wallabag.api.get_entry import GetEntry
from wallabag.api.get_list_entries import GetListEntries, Params as ListParams
from wallabag.api.get_tags_for_entry import GetTagsForEntry
from wallabag.api.delete_tag_by_id import DeleteTagsById
from wallabag.api.delete_tags_by_label import DeleteTagsByLabel
from wallabag.commands.command import Command
from wallabag.commands.tags_param import TagsParam
from wallabag.commands.params import Params
from wallabag.entry import Entry
from wallabag import wclick


class TagsSubcommand(Enum):
    LIST = auto()
    ADD = auto()
    REMOVE = auto()

    def list():
        return [c.name for c in TagsSubcommand]

    def get(name):
        for command in TagsSubcommand:
            if command.name == name.upper():
                return command
        return TagsSubcommand.LIST


class RemoveSubcommand(Enum):
    FROM_ENTRY = auto()
    BY_TAG_ID = auto()
    BY_TAG_NAME = auto()


class TagsCommandParams(Params, TagsParam):
    command = TagsSubcommand.LIST
    remove_command = None
    entry_id = None
    tag_id = None
    tags = None
    validate_tags = False
    validate_entry_id = False
    validate_tag_id = False

    config = {TagsSubcommand.LIST: (False, False, False),
              TagsSubcommand.ADD: (True, True, False),
              TagsSubcommand.REMOVE: {
                  RemoveSubcommand.FROM_ENTRY: (True, True, False),
                  RemoveSubcommand.BY_TAG_ID: (False, False, True),
                  RemoveSubcommand.BY_TAG_NAME: (True, False, False)
              }}

    def __init__(self, entry_id=None, tags=None, tag_id=None):
        self.entry_id = entry_id
        self.tags = tags
        self.tag_id = tag_id

    def configure(self, command):
        self.command = command
        result = self.config[command]
        if self.command == TagsSubcommand.REMOVE:
            if self.entry_id:
                self.remove_command = RemoveSubcommand.FROM_ENTRY
            elif self.tag_id:
                self.remove_command = RemoveSubcommand.BY_TAG_ID
            else:
                self.remove_command = RemoveSubcommand.BY_TAG_NAME
            result = result[self.remove_command]

        self.validate_tags = result[0]
        self.validate_entry_id = result[1]
        self.validate_tag_id = result[2]

    def validate(self, tags=False, entry_id=False, tag_id=False):
        if self.validate_tags:
            result, msg = self._validate_tags()
            if not result:
                return False, msg
        if self.validate_entry_id and not self.entry_id:
            return False, 'Entry id not specified'
        if self.validate_tag_id:
            if self.tag_id:
                try:
                    self.tag_id = int(self.tag_id)
                except ValueError:
                    return False, 'Tag id is not integer'
            else:
                return False, 'Tag id is not set'
        return True, None


class TagsCommand(Command):

    def __init__(self, config, params=None):
        Command.__init__(self)
        self.config = config
        self.params = params if params else TagsCommandParams()

        self.runner = {
            TagsSubcommand.LIST: self.__subcommand_list,
            TagsSubcommand.ADD: self.__subcommand_add,
            TagsSubcommand.REMOVE: self.__subcommand_remove
        }

    def _run(self):
        return self.runner[self.params.command]()

    def __subcommand_list(self):
        if self.params.entry_id:
            api = GetTagsForEntry(self.config, self.params.entry_id)
        else:
            api = GetTags(self.config)

        return True, self.__parse_tags(api.request().response)

    def __subcommand_add(self):
        AddTagToEntry(self.config, {
            AddTagParams.ENTRY_ID: self.params.entry_id,
            AddTagParams.TAGS: self.params.tags
        }).request()

        return True, 'Tags successfully added'

    def __subcommand_remove(self):
        remove = {
                RemoveSubcommand.FROM_ENTRY: self.__remove_from_entry,
                RemoveSubcommand.BY_TAG_ID: self.__remove_by_tag_id,
                RemoveSubcommand.BY_TAG_NAME: self.__remove_by_tag_name
        }
        if self.params.remove_command not in remove:
            return False, 'Command not found'
        return remove[self.params.remove_command]()

    def __remove_by_tag_id(self):
        confirm_msg = (
                f'{Back.RED}You are going to remove tag with id: '
                f'{Fore.BLUE}{self.params.tag_id}{Fore.RESET}{Back.RESET}'
                '\n\nContinue?')
        if not wclick.confirm(confirm_msg):
            return True, 'Cancelling'

        DeleteTagsById(self.config, self.params.tag_id).request()
        return True, None

    def __remove_by_tag_name(self):
        entries = list()
        for tag in self.params.tags.split(','):
            api = GetListEntries(self.config, {
                ListParams.TAGS: tag,
                ListParams.COUNT: 100
            })
            entries.extend(Entry.create_list(
                    api.request().response['_embedded']["items"]))
        titles = "\n\t".join([x.title for x in entries])
        confirm_msg = (
                f'{Back.RED}You are going to remove tag '
                f'{Fore.BLUE}{self.params.tags}{Fore.RESET} '
                f'from this entries:{Back.RESET}\n\n\t{titles}'
                '\n\nContinue?')
        if not wclick.confirm(confirm_msg):
            return True, 'Cancelling'

        DeleteTagsByLabel(self.config, self.params.tags).request()
        return True, None

    def __remove_from_entry(self):
        api = GetEntry(self.config, self.params.entry_id)
        entry = Entry(api.request().response)
        tag = list(filter(
                lambda t: t['slug'] == self.params.tags,
                entry.tags))
        if not tag:
            return False, (
                    f'Tag "{self.params.tags}" not found '
                    f'in entry:\n\n\t{entry.title}\n')
        tag = tag[0]

        confirm_msg = (
                f'{Back.RED}You are going to remove tag '
                f'{Fore.BLUE}{self.params.tags}{Fore.RESET} from entry:'
                f'{Back.RESET}'
                f'\n\n\t{entry.title}\n\n'
                'Continue?')
        if not wclick.confirm(confirm_msg):
            return True, 'Cancelling'

        DeleteTagFromEntry(
                self.config, self.params.entry_id,
                tag['id']).request()
        return True, None

    def __parse_tags(self, response):
        def sort(tag):
            return int(tag['id'])

        return "\n".join(
                map(
                    lambda tag: self.__tag_format(tag),
                    sorted(response, key=sort)))

    def __tag_format(self, tag):
        return f"{tag['id']}. {tag['slug']}"



# -*- coding: utf-8 -*-

import webbrowser

from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.api.get_entry import GetEntry
from wallabag.entry import Entry
from wallabag.config import Sections, Options


class OpenCommandParams(Params):
    entry_id = None
    open_original = False
    browser = None

    def __init__(self, entry_id, open_original=False, browser=None):
        self.entry_id = entry_id
        self.open_original = open_original
        self.browser = browser

    def validate(self):
        if not self.entry_id:
            return False, 'Entry ID not specified'
        return True, None


class OpenCommand(Command):

    def __init__(self, config, params):
        Command.__init__(self)
        self.config = config
        self.params = params if params else OpenCommandParams(None)

    def _run(self):
        api = GetEntry(self.config, self.params.entry_id)
        entry = Entry(api.request().response)

        url = ""
        if self.params.open_original:
            url = entry.url
        else:
            serverurl = self.config.get(Sections.API, Options.SERVERURL)
            url = f"{serverurl}/view/{entry.entry_id}"

        browser = webbrowser.get(self.params.browser)
        browser.open_new_tab(url)

        return True, None



# -*- coding: utf-8 -*-

import click
from colorama import Fore, Back

from wallabag.api.get_list_entries import GetListEntries, Params as ListParams
from wallabag.commands.command import Command
from wallabag.commands.update import UpdateCommandParams
from wallabag.api.update_entry import UpdateEntry, Params
from wallabag.entry import Entry
from wallabag import wclick


class UpdateByTagsCommand(Command):

    def __init__(self, config, tags, params):
        Command.__init__(self)
        self.config = config
        self.tags = tags
        self.params = params or UpdateCommandParams(False)

        self.__updating_list = []

    def _run(self):
        params = self.params

        api = GetListEntries(self.config, {
            ListParams.TAGS: self.tags,
            ListParams.COUNT: 100
        })
        entries = Entry.create_list(
                api.request().response['_embedded']["items"])
        if not params.force:
            titles = "\n\t".join([x.title for x in entries])
            confirm_msg = (
                    f'{Back.GREEN}You are going to update '
                    f'{self.__get_update_status()} '
                    f'of followed entries:{Back.RESET}'
                    f'\n\n\t{titles}\n\nContinue?')
            if not wclick.confirm(confirm_msg):
                return True, 'Cancelling'

        for entry in entries:
            self.__log(entry.title)
            UpdateEntry(self.config, entry.entry_id, {
                Params.STAR: self.params.set_star_state,
                Params.READ: self.params.set_read_state
            }).request()
            self.__log(entry.title)

        return True, None

    def __get_update_status(self):
        result = ""
        if self.params.set_read_state is not None:
            result = "read status to "
            if self.params.set_read_state:
                result += "read"
            else:
                result += "not read"
        if self.params.set_star_state is not None:
            if result:
                result += " and "
            result += "starred status to "
            if self.params.set_star_state:
                result += "starred"
            else:
                result += "unstarred"
        return result

    def __log(self, title):
        if self.params.quiet:
            return
        if title not in self.__updating_list:
            click.echo(f'Updating entry: {title}', nl=False)
            self.__updating_list.append(title)
        else:
            click.echo(f'\t...\t{Fore.GREEN}success{Fore.RESET}')



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.add_tag_to_entry import AddTagToEntry, Params
from wallabag.config import Configs


def _is_valid_url(self, url):
    return True


class TestAddEntry():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    @pytest.mark.parametrize('entry_id', [10, "10", ''])
    def test_api_url(self, entry_id):
        api = AddTagToEntry(self.config, {Params.ENTRY_ID: entry_id})

        url = "url" + ApiMethod.ADD_TAGS_TO_ENTRY.value.format(entry_id)
        if not entry_id:
            with pytest.raises(ValueException):
                api._get_api_url()
        else:
            assert url == api._get_api_url()

    def test_params(self, monkeypatch):
        monkeypatch.setattr(AddTagToEntry, '_is_valid_url', _is_valid_url)

        api = AddTagToEntry(self.config, {
            Params.ENTRY_ID: 10,
            Params.TAGS: 'tag1,tag2'
        })

        data = api._get_data()
        assert AddTagToEntry.ApiParams.TAGS.value in data



# -*- coding: utf-8 -*-

import os
import sys

import tabulate

from colorama import Fore

from wallabag.api.get_list_entries import (
        GetListEntries, Params as ListEntriesParams)
from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.commands.tags_param import TagsParam
from wallabag.entry import Entry


class ListParams(Params, TagsParam):
    quantity = None
    filter_read = None
    filter_starred = None
    oldest = False
    trim = False
    tags = None

    def __init__(self, quantity=None, filter_read=None,
                 filter_starred=None, oldest=None, trim=None, tags=None):
        self.quantity = quantity
        self.filter_read = filter_read
        self.filter_starred = filter_starred
        self.oldest = oldest
        self.trim = trim
        self.tags = tags

    def validate(self):
        result, msg = self._validate_tags()
        if not result:
            return False, msg
        return True, None


class ListCommand(Command):

    def __init__(self, config, params=None):
        Command.__init__(self)
        self.config = config
        self.params = params or ListParams()

    def _run(self):
        api = GetListEntries(self.config, {
            ListEntriesParams.COUNT: self.__get_quantity(),
            ListEntriesParams.FILTER_READ: self.params.filter_read,
            ListEntriesParams.FILTER_STARRED: self.params.filter_starred,
            ListEntriesParams.OLDEST: self.params.oldest,
            ListEntriesParams.TAGS: self.params.tags
        })
        entries = Entry.create_list(
                api.request().response['_embedded']["items"])
        return True, self.__print_entries(entries)

    def __print_entries(self, entries):
        entry_id_width = self.__entry_id_width(entries)
        maxwidth = self.__get_maxwidth() - entry_id_width - 4
        output = []
        for item in entries:
            entry_id = str(item.entry_id).rjust(entry_id_width)

            title = item.title
            status = self.__read_star_char(item)
            annotation_mark = self.__annotation_mark(item)
            tags_mark = self.__tags_mark(item)
            entry = [
                    entry_id, status,
                    f"{title}{annotation_mark}{tags_mark}"[0:maxwidth]]
            output.append(entry)
        return tabulate.tabulate(output)

    def __get_quantity(self):
        if self.params.quantity is None:
            try:
                return os.get_terminal_size().lines - 2
            except OSError:
                return sys.maxsize
        else:
            return self.params.quantity

    def __get_maxwidth(self):
        if self.params.trim:
            try:
                return os.get_terminal_size().columns
            except OSError:
                pass
        return sys.maxsize

    def __entry_id_width(self, entries):
        if len(entries) > 0:
            size_entry_id = len(str(entries[0].entry_id))
            entry_id_last = len(str(entries[len(entries) - 1].entry_id))
            if entry_id_last > size_entry_id:
                size_entry_id = entry_id_last
            return size_entry_id
        return 0

    def __read_star_char(self, item):
        if item.starred:
            return "" if not item.read else ""
        return "" if not item.read else ""

    def __annotation_mark(self, entry):
        return f" {Fore.BLUE}{Fore.RESET}" if entry.annotations else ""

    def __tags_mark(self, entry):
        return f" {Fore.BLUE}{Fore.RESET}" if entry.tags else ""


class CountCommand(Command):

    def __init__(self, config, params=None):
        self.config = config
        self.params = params or ListParams()

    def _run(self):
        api = GetListEntries(self.config, {
            ListEntriesParams.COUNT: sys.maxsize,
            ListEntriesParams.FILTER_READ: self.params.filter_read,
            ListEntriesParams.FILTER_STARRED: self.params.filter_starred
        })
        response = api.request().response

        return True, len(response["_embedded"]["items"])



# -*- coding: utf-8 -*-

import logging
import re
import time
from abc import ABC, abstractmethod

from wallabag.api.api import (
        Api, Error, MINIMUM_API_VERSION, RequestException)
from wallabag.api.get_api_version import ApiVersion
from wallabag.api.api_token import ApiToken
from wallabag.commands.command import Command
from wallabag.config import Options, Sections
from wallabag import wclick


class Configurator():

    def __init__(self, config):
        self.log = logging.getLogger('wallabag.config')
        self.config = config

    def start(self, options=[]):
        if not options:
            options = ConfigOption.get_all(self.config)
        for option in options:
            self.log.debug('configuring: %s', option.__class__.__name__)

            while not option.setup(self.config):
                pass

        self.config.save()


class Validator(Command):

    def __init__(self, config):
        Command.__init__(self)
        self.config = config
        self.response = {
            'invalid_grant': (
                False, None, [UsernameOption(), PasswordOption()]),
            'invalid_client': (
                False, None, [ClientOption(), SecretOption()])
        }

    def check_oauth(self):
        try:
            ApiToken(self.config).request()
        except RequestException as error:
            response = error.response
            if response and response.error == Error.HTTP_BAD_REQUEST:
                wclick.echo(response.error_description)
                return self.response[response.error_text]
            return (False, error.error_description, None)
        return (True, "The configuration is ok.", None)

    def _run(self):
        if not self.config.is_valid():
            return False, "The config is missing or incomplete."

        try:
            response = ApiVersion(self.config).request()
        except RequestException:
            return False, "The server or the API is not reachable."

        if not Api.is_minimum_version(response):
            return False, "The version of the wallabag instance is too old."

        ApiToken(self.config).request()

        return True, "The config is suitable."


class ConfigOption(ABC):

    prompt = ''
    default = ''

    def get_all(config):
        return [ServerurlOption(config), UsernameOption(),
                PasswordOption(), ClientOption(), SecretOption()]

    def __init__(self, default=None):
        self.set_default(default)

    def _check_existence_or_default(self, value):
        if not value:
            if not self.get_default():
                raise ValueError("Empty value")
            return self.get_default()
        return value

    def get_prompt(self):
        return self.prompt

    def get_default(self):
        return self.default

    def set_default(self, default):
        if default:
            self.default = default

    @abstractmethod
    def get_option_name(self):
        pass

    def check_and_apply(self, value):
        self.value = self._check_existence_or_default(value.strip())

    def get_value(self):
        return self.value

    def setup(self, config):
        (sec, opt) = self.get_option_name()
        self.set_default(config.get(sec, opt))
        value = wclick.prompt(self.get_prompt(),
                              default=self.get_default())
        try:
            self.check_and_apply(value)
        except ValueError as e:
            wclick.echo(e)
            return False
        config.set(sec, opt, self.get_value())
        return True


class ServerurlOption(ConfigOption):

    RE_HTTP = re.compile("(?i)https?:\\/\\/.+")

    prompt = 'Enter the url of your Wallabag instance'
    default = 'https://www.wallabag.com/'

    def __init__(self, config):
        self.config = config

    def __check_trailing_space(self, value):
        return value[:-1] if value[-1] == '/' else value

    def __check_https(self, value):
        return f"https://{value}" if not ServerurlOption.RE_HTTP.match(value)\
                else value

    def __check_api_verion(self, value):
        try:
            response = ApiVersion(self.config, value).request()
        except RequestException as err:
            raise ValueError(str(err))

        if not Api.is_minimum_version(response):
            raise ValueError(
                    f"Your wallabag instance is too old. \
                            You need at least version \
                            {MINIMUM_API_VERSION}.")

    def check_and_apply(self, value):
        value = self._check_existence_or_default(value.strip())
        value = self.__check_trailing_space(value)
        value = self.__check_https(value)
        self.__check_api_verion(value)
        self.value = value

    def get_option_name(self):
        return (Sections.API, Options.SERVERURL)


class UsernameOption(ConfigOption):

    prompt = 'Enter your Wallabag username'

    def get_option_name(self):
        return (Sections.API, Options.USERNAME)


class PasswordOption(ConfigOption):

    prompt = 'Enter your Wallabag password'

    def get_option_name(self):
        return (Sections.API, Options.PASSWORD)


class ClientOption(ConfigOption):

    prompt = 'Enter the client id of your Wallabag API'

    def get_option_name(self):
        return (Sections.OAUTH2, Options.CLIENT)


class SecretOption(ConfigOption):

    prompt = 'Enter the client secret of your Wallabag API'

    def get_option_name(self):
        return (Sections.OAUTH2, Options.SECRET)


class TokenConfigurator():

    def __init__(self, config):
        self.config = config

    def get_token(self, force_creation=False):
        if self.config.is_token_expired() or force_creation:
            response = ApiToken(self.config).request()
            content = response.response
            self.config.set(
                    Sections.TOKEN,
                    Options.ACCESS_TOKEN,
                    content['access_token'])
            self.config.set(
                    Sections.TOKEN,
                    Options.EXPIRES,
                    str(time.time() + content['expires_in']))
            self.config.save()
            return self.config.get(
                    Sections.TOKEN,
                    Options.ACCESS_TOKEN)
        else:
            return self.config.get(
                    Sections.TOKEN,
                    Options.ACCESS_TOKEN)



# -*- coding: utf-8 -*-

from colorama import Fore, Back

from wallabag.commands.tags_param import TagsParam
from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.api.get_list_entries import (
        GetListEntries, Params as ListEntriesParams)
from wallabag.api.delete_entry import DeleteEntry
from wallabag.entry import Entry
from wallabag import wclick


class DeleteByTagsParams(Params, TagsParam):
    tags = None
    force = None
    quiet = None

    def __init__(self, tags, force=False, quiet=False):
        self.tags = tags
        self.force = force
        self.quiet = quiet

    def validate(self):
        return self._validate_tags()


class DeleteByTags(Command):

    def __init__(self, config, params=None):
        Command.__init__(self)
        self.config = config
        self.params = params if params else DeleteByTagsParams()

    def _run(self):
        api = GetListEntries(self.config, {
            ListEntriesParams.TAGS: self.params.tags,
            ListEntriesParams.COUNT: 100
        })
        entries = Entry.create_list(
                api.request().response['_embedded']["items"])
        if not self.params.force:
            titles = "\n\t".join([x.title for x in entries])
            confirm_msg = (
                    f'{Back.RED}You are going to remove '
                    f'followed entries:{Back.RESET}'
                    f'\n\n\t{titles}\n\nContinue?')
            if not wclick.confirm(confirm_msg):
                return True, 'Cancelling'

        for entry in entries:
            if not self.params.quiet:
                wclick.echo(f'Deleting entry: {entry.title}', nl=False)
            DeleteEntry(self.config, entry.entry_id).request()
            if not self.params.quiet:
                wclick.echo(f'\t...\t{Fore.GREEN}success{Fore.RESET}')

        return True, None



# -*- coding: utf-8 -*-

from wallabag.api.api import Api, ApiMethod


class DeleteTagsById(Api):

    def __init__(self, config, tag_id):
        Api.__init__(self, config)
        self.tag_id = tag_id

    def _get_api_url(self):
        return self._build_url(
                ApiMethod.DELETE_TAG_BY_ID).format(self.tag_id)

    def _make_request(self, request):
        return self._request_delete(request)



# -*- coding: utf-8 -*-

from enum import Enum

from wallabag.api.api import Api, ApiMethod


class DeleteTagsByLabel(Api):

    class ApiParams(Enum):
        TAGS = "tags"

    def __init__(self, config, tags):
        Api.__init__(self, config)
        self.tags = tags

    def _get_api_url(self):
        return self._build_url(
                ApiMethod.DELETE_TAG_BY_LABEL)

    def _make_request(self, request):
        return self._request_delete(request)

    def _get_params(self):
        return {
            DeleteTagsByLabel.ApiParams.TAGS.value: self.tags
        }



# -*- coding: utf-8 -*-


class TagsParam():

    def _validate_tags(self):
        if self.tags is not None:
            used = set()
            self.tags = ",".join([x.strip() for x in self.tags.split(',')
                                  if x.strip() and x not in used and
                                  (used.add(x) or True)])
            if not self.tags:
                return False, 'tags value is empty'
        return True, None



# -*- coding: utf-8 -*-

from wallabag.api.api import ApiMethod
from wallabag.api.get_tags import GetTags
from wallabag.config import Configs


class TestGetTags():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_api_url(self):
        api = GetTags(self.config)

        url = "url" + ApiMethod.GET_TAGS.value
        assert url == api._get_api_url()



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.delete_entry import DeleteEntry
from wallabag.config import Configs


class TestDeleteEntry():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    @pytest.mark.parametrize('entry_id', [10, "10"])
    def test_api_url(self, entry_id):
        api = DeleteEntry(self.config, entry_id)

        url = "url" + ApiMethod.DELETE_ENTRY.value.format(entry_id)
        assert url == api._get_api_url()

    @pytest.mark.parametrize('entry_id', ["-10", -10, None, "none"])
    def test_api_url_wrong_arg(self, entry_id):
        api = DeleteEntry(self.config, entry_id)

        with pytest.raises(ValueException):
            api._get_api_url()



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='wallabag-client',
    use_scm_version=True,
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/artur-shaik/wallabag-client',
    author='Artur Shaik',
    author_email='artur@shaik.link',
    description=('A command-line client for the self-hosted '
                 '`read-it-later` app Wallabag'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',

    setup_requires=[
        'pytest-runner==5.1',
        'setuptools_scm==3.3.3',
    ],
    install_requires=[
        'beautifulsoup4>=4.9.1',
        'pycryptodome>=3.9.8',
        'requests>=2.11.1',
        'click>=7.0',
        'click_spinner',
        'pyxdg',
        'colorama>=0.4.3',
        'delorean',
        'humanize',
        'lxml',
        'tzlocal<3',
        'tabulate'
    ],
    tests_require=[
        'pytest==4.6.3',
    ],

    entry_points='''
        [console_scripts]
        wallabag=wallabag.wallabag:cli
    '''
)



# -*- coding: utf-8 -*-

import delorean


class Entry:
    entry_id = 0
    title = ""
    content = ""
    url = ""
    read = False
    starred = False
    created_at = None
    reading_time = None
    preview_picture = None
    published_by = []
    tags = []
    annotations = []

    def __init__(self, item):
        self.entry_id = item['id']

        title = item['title']
        title = title.replace("\n", "")
        title = " ".join(title.split())
        self.title = title

        self.content = item['content']
        self.url = item['url']
        self.read = item['is_archived'] == 1
        self.starred = item['is_starred'] == 1
        if 'created_at' in item:
            self.created_at = delorean.parse(item['created_at'])
        if 'reading_time' in item:
            self.reading_time = item['reading_time']
        if 'preview_picture' in item:
            self.preview_picture = item['preview_picture']
        if 'published_by' in item:
            self.published_by = item['published_by']
        if 'tags' in item:
            self.tags = item['tags']
        if 'annotations' in item:
            self.annotations = item['annotations']

    def get_tags_string(self):
        if not self.tags:
            return None

        def sort(tag):
            return int(tag['id'])

        return " ".join(
                map(
                    lambda tag: tag['label'],
                    sorted(self.tags, key=sort)))

    def create_list(items):
        return [Entry(i) for i in items]



# -*- coding: utf-8 -*-

from enum import Enum

from wallabag.api.api import Api, ApiMethod, ValueException


class Params(Enum):
    ENTRY_ID = "entry_id"
    TAGS = "tags"


class AddTagToEntry(Api):

    class ApiParams(Enum):
        TAGS = "tags"

    def __init__(self, config, params):
        Api.__init__(self, config)
        self.params = params

    def _make_request(self, request):
        return self._request_post(request)

    def _get_api_url(self):
        entry_id = self._validate_identificator(self.params[Params.ENTRY_ID])
        return self._build_url(ApiMethod.ADD_TAGS_TO_ENTRY).format(entry_id)

    def _get_data(self):
        if Params.TAGS not in self.params:
            raise ValueException(f"{Params.TAGS.value} is empty")
        return {
            AddTagToEntry.ApiParams.TAGS.value: self.params[Params.TAGS]
        }



# -*- coding: utf-8 -*-

import pytest
import click

from colorama import Fore, Back

from tags import tags_test
from wallabag.config import Configs
from wallabag.api.add_tag_to_entry import AddTagToEntry
from wallabag.api.api import Response
from wallabag.api.api import RequestException
from wallabag.api.delete_tag_by_id import DeleteTagsById
from wallabag.api.delete_tags_by_label import DeleteTagsByLabel
from wallabag.api.delete_tag_from_entry import DeleteTagFromEntry
from wallabag.api.get_entry import GetEntry
from wallabag.api.get_list_entries import GetListEntries
from wallabag.api.get_tags import GetTags
from wallabag.api.get_tags_for_entry import GetTagsForEntry
from wallabag.commands.tags import (
        TagsCommand, TagsCommandParams, TagsSubcommand)


class TestTags():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_list_tags(self, monkeypatch):

        def request(self):
            return Response(200, """[
            {"id": 3, "label": "book", "slug": "book"},
            {"id": 8, "label": "gut", "slug": "gut"},
            {"id": 7, "label": "security", "slug": "security"}]
            """)

        monkeypatch.setattr(GetTags, 'request', request)

        result, msg = TagsCommand(self.config).execute()
        assert result
        assert msg == '3. book\n7. security\n8. gut'

    def test_empty_tag_list(self, monkeypatch):

        def request(self):
            return Response(200, """[]""")

        monkeypatch.setattr(GetTags, 'request', request)

        result, msg = TagsCommand(self.config).execute()
        assert result
        assert not msg

    def test_list_entry_tags(self, monkeypatch):
        def request(self):
            return Response(200, """[
            {"id": 3, "label": "book", "slug": "book"},
            {"id": 8, "label": "gut", "slug": "gut"},
            {"id": 7, "label": "security", "slug": "security"}]
            """)

        monkeypatch.setattr(GetTagsForEntry, 'request', request)

        params = TagsCommandParams(entry_id=10)
        result, msg = TagsCommand(self.config, params).execute()
        assert result
        assert msg == '3. book\n7. security\n8. gut'

    def test_entry_not_found(self, monkeypatch):

        def request(self):
            response = Response(404, None)
            raise RequestException(
                    response.error_text, response.error_description)

        monkeypatch.setattr(GetTagsForEntry, 'request', request)

        params = TagsCommandParams(entry_id=10)
        result, msg = TagsCommand(self.config, params).execute()
        assert not result

    def test_add_tags_to_entry(self, monkeypatch):
        tag_to_entry_runned = False

        def request(self):
            nonlocal tag_to_entry_runned
            tag_to_entry_runned = True
            return Response(200, """[]""")

        monkeypatch.setattr(AddTagToEntry, 'request', request)

        params = TagsCommandParams(entry_id=10, tags='tag1')
        params.command = TagsSubcommand.ADD
        result, msg = TagsCommand(self.config, params).execute()
        assert tag_to_entry_runned
        assert result
        assert msg == 'Tags successfully added'

    @pytest.mark.parametrize('tags', tags_test)
    def test_tags_param(self, monkeypatch, tags):
        make_request_runned = False

        def request(self):
            nonlocal make_request_runned
            make_request_runned = True
            return Response(200, """[]""")

        monkeypatch.setattr(AddTagToEntry, 'request', request)

        params = TagsCommandParams(entry_id=10, tags=tags[0])
        params.configure(TagsSubcommand.ADD)
        result = TagsCommand(self.config, params).execute()
        if tags[0]:
            assert make_request_runned
            assert result[0]
            assert result[1] == "Tags successfully added"
        else:
            assert not make_request_runned
            assert not result[0]
            assert result[1] == "tags value is empty"

    def test_tags_to_entry_empty_id(self):
        params = TagsCommandParams(tags='tags')
        params.configure(TagsSubcommand.ADD)
        result = TagsCommand(self.config, params).execute()
        assert not result[0]
        assert result[1] == 'Entry id not specified'

    def test_remove_tag_from_entry_not_found(self, monkeypatch):

        def getentry_request(self):
            return Response(
                    200, (
                        '{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"tags": ['
                        '{"id":7,"label":"tag1","slug":"tag1"},'
                        '{"id":13,"label":"tag2","slug":"tag2"}]}'))

        monkeypatch.setattr(GetEntry, 'request', getentry_request)

        params = TagsCommandParams(tags='tag', entry_id=1)
        params.configure(TagsSubcommand.REMOVE)
        result = TagsCommand(self.config, params).execute()
        assert not result[0]
        assert result[1] == 'Tag "tag" not found in entry:\n\n\ttitle\n'

    def test_remove_tag_from_entry(self, monkeypatch):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def getentry_request(self):
            return Response(
                    200, (
                        '{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"tags": ['
                        '{"id":7,"label":"tag","slug":"tag"},'
                        '{"id":13,"label":"tag2","slug":"tag2"}]}'))

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            assert msg == (
                    f'{Back.RED}You are going to remove tag '
                    f'{Fore.BLUE}tag{Fore.RESET} from entry:'
                    f'{Back.RESET}\n\n\ttitle\n\nContinue?')

        monkeypatch.setattr(GetEntry, 'request', getentry_request)
        monkeypatch.setattr(DeleteTagFromEntry, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = TagsCommandParams(tags='tag', entry_id=1)
        params.configure(TagsSubcommand.REMOVE)
        result = TagsCommand(self.config, params).execute()
        assert confirm_runned
        assert result[0]

    @pytest.mark.parametrize('current_tag', ['tag', 'tag2'])
    def test_remove_tag_by_label(self, monkeypatch, current_tag):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def getlist_request(self):
            return Response(
                    200, (
                        '{"_embedded": {"items": [{"id": 1, "title": "title",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"tags": ['
                        '{"id":7,"label":"tag","slug":"tag"},'
                        '{"id":13,"label":"tag2","slug":"tag2"}]},'
                        '{"id": 2, "title": "title 2",'
                        '"content": "<h1>head</h1>content", "url": "url",'
                        '"is_archived": 0, "is_starred": 1,'
                        '"tags": ['
                        '{"id":7,"label":"tag","slug":"tag"},'
                        '{"id":13,"label":"tag2","slug":"tag2"}]}]}}'))

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            assert msg == (
                    f'{Back.RED}You are going to remove tag '
                    f'{Fore.BLUE}{current_tag}{Fore.RESET} from this entries:'
                    f'{Back.RESET}\n\n\ttitle\n\ttitle 2\n\nContinue?')
            return True

        monkeypatch.setattr(GetListEntries, 'request', getlist_request)
        monkeypatch.setattr(DeleteTagsByLabel, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = TagsCommandParams(tags=current_tag)
        params.configure(TagsSubcommand.REMOVE)
        result = TagsCommand(self.config, params).execute()
        assert confirm_runned
        assert result[0]

    @pytest.mark.parametrize('tag_id', [1, '2'])
    def test_remove_tag_by_id(self, monkeypatch, tag_id):
        confirm_runned = False

        def success(self):
            return Response(200, None)

        def confirm(msg):
            nonlocal confirm_runned
            confirm_runned = True
            assert msg == (
                    f'{Back.RED}You are going to remove tag with id: '
                    f'{Fore.BLUE}{int(tag_id)}{Fore.RESET}{Back.RESET}'
                    '\n\nContinue?')
            return True

        monkeypatch.setattr(DeleteTagsById, 'request', success)
        monkeypatch.setattr(click, 'confirm', confirm)

        params = TagsCommandParams(tag_id=tag_id)
        params.configure(TagsSubcommand.REMOVE)
        result = TagsCommand(self.config, params).execute()
        assert confirm_runned
        assert result[0]



# -*- coding: utf-8 -*-

from wallabag.api.api import Api, ApiMethod


class GetTags(Api):

    def __init__(self, config):
        Api.__init__(self, config)

    def _get_api_url(self):
        return self._build_url(ApiMethod.GET_TAGS)

    def _make_request(self, request):
        return self._request_get(request)



# -*- coding: utf-8 -*-

from colorama import Fore

from wallabag.commands.command import Command
from wallabag.commands.params import Params
from wallabag.entry import Entry
from wallabag.api.get_entry import GetEntry


class InfoCommandParams(Params):
    entry_id = None

    def __init__(self, entry_id):
        self.entry_id = entry_id

    def validate(self):
        if not self.entry_id:
            return False, 'Entry ID not specified'
        return True, None


class InfoCommand(Command):

    def __init__(self, config, params):
        Command.__init__(self)
        self.config = config
        self.params = params if params else InfoCommandParams(None)

    def _run(self):
        entry = Entry(
                GetEntry(
                    self.config, self.params.entry_id).request().response)
        return True, self.__entry_to_string(entry)

    def __entry_to_string(self, entry):
        f_c = Fore.LIGHTBLUE_EX
        f_rst = Fore.RESET

        published_by = ""
        if entry.published_by:
            published_by = (
                    f'{f_c}Published by{f_rst}: '
                    f'{",".join(entry.published_by)}\n')
        preview_picture = ""
        if entry.preview_picture:
            preview_picture = (
                    f'{f_c}Preview picture{f_rst}: '
                    f'{entry.preview_picture}')

        tags = ""
        if entry.tags:
            tags = f'{f_c}Tags{f_rst}: {entry.get_tags_string()}\n'

        annotations = ""
        if entry.annotations:
            annotations =\
                f'{f_c}Annotations{f_rst}: {len(entry.annotations)}\n'

        return (
                f'{f_c}ID{f_rst}: {entry.entry_id}\n'
                f'{f_c}Title{f_rst}: {entry.title}\n'
                f'{f_c}Url{f_rst}: {entry.url}\n'
                f'{annotations}'
                f'{tags}'
                f'{f_c}Is read{f_rst}: {entry.read}\n'
                f'{f_c}Is starred{f_rst}: {entry.starred}\n'
                f'{f_c}Created at{f_rst}: '
                f'{entry.created_at.format_datetime()}\n'
                f'{published_by}'
                f'{f_c}Reading time{f_rst}: {entry.reading_time} min\n'
                f'{preview_picture}')
